package com.chinaitop.depot.profitlossStatement.controller;

import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.profitlossStatement.service.DataStatisticsService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value="/DataStatisticsController")
@Api(value= "DataStatisticsController", description = "损溢单数据统计")
public class DataStatisticsController {

	final static Logger logger = LoggerFactory.getLogger(DataStatisticsController.class);

	@Resource
	private DataStatisticsService dataStatisticsService;

	@RequestMapping(value="/getPageInfoCflx",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="按仓房类型统计损益单数据列表", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "出库时间(起始)", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "出库时间(终止)", paramType = "query"),
            @ApiImplicitParam(name = "orgCode", value = "市平台库区编码", paramType = "query")
    })
	public Map<String, Object> getPageInfoCflx(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) {
		Map<String, Object> map = new HashMap<>();
		try {
			map = dataStatisticsService.selectByCflx(pageNum, pageSize, startTime, endTime, orgCode);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

	@RequestMapping(value="/getPageInfoLspz",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="按粮食品种统计损益单数据列表", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "出库时间(起始)", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "出库时间(终止)", paramType = "query"),
            @ApiImplicitParam(name = "orgCode", value = "市平台库区编码", paramType = "query")
    })
	public Map<String, Object> getPageInfoLspz(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) {
		Map<String, Object> map = new HashMap<>();
		try {
			map = dataStatisticsService.selectByLspz(pageNum, pageSize, startTime, endTime, orgCode);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

	@RequestMapping(value="/getPageInfoLsxz",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="按粮食性质统计损益单数据列表", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "出库时间(起始)", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "出库时间(终止)", paramType = "query"),
            @ApiImplicitParam(name = "orgCode", value = "市平台库区编码", paramType = "query")
    })
	public Map<String, Object> getPageInfoLsxz(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) {
		Map<String, Object> map = new HashMap<>();
		try {
			map = dataStatisticsService.selectByLsxz(pageNum, pageSize, startTime, endTime, orgCode);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

	@RequestMapping(value="/getPageInfoKdmc",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="按库点名称统计损益单数据列表", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "出库时间(起始)", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "出库时间(终止)", paramType = "query"),
            @ApiImplicitParam(name = "orgCode", value = "市平台库区编码", paramType = "query")
    })
	public Map<String, Object> getPageInfoKdmc(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) {
		Map<String, Object> map = new HashMap<>();
		try {
			map = dataStatisticsService.selectByKdmc(pageNum, pageSize, startTime, endTime, orgCode);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

	@RequestMapping(value="/exportData", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="条件导出入库情况数据", notes = "条件查询和列表一致")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "startTime", value = "出库时间(起始)", paramType = "query"),
        @ApiImplicitParam(name = "endTime", value = "出库时间(终止)", paramType = "query"),
        @ApiImplicitParam(name = "orgCode", value = "市平台库区编码", paramType = "query")
	})
	public void exportData(HttpServletResponse response,HttpServletRequest request, String startTime, String endTime, String orgCode){
		try {
			HSSFWorkbook workbook = dataStatisticsService.exportData(startTime, endTime, orgCode);
			String fileName = "分析汇总表导出";
			// 获取输出流
			OutputStream os = response.getOutputStream();
			// 重置输出流
			response.reset();
			// 设定输出文件头
			response.setHeader("Content-disposition", "attachment; filename=" + new String(fileName.getBytes("GB2312"), "8859_1") + ".xlsx");
			//name属性设置和跨服访问有关
			response.setHeader("Access-Control-Allow-Origin", "*");
			// 定义输出内容类型
			response.setContentType("application/vnd.ms-excel;charset=gb2312");
			workbook.write(os);
			//关闭流
			os.close();
			return;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}
}
