package com.chinaitop.depot.profitlossStatement.controller;


import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossStatement;
import com.chinaitop.depot.profitlossStatement.service.StorageProfitlossStatementService;
import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 损溢单
 * @author My Sunshine
 *
 */
@RestController
@RequestMapping(value="/storageProfitlossStatement")
@Api(value= "StorageProfitlossStatementController", description = "损溢单申请")
public class StorageProStatementController {

    @Resource
    private StorageProfitlossStatementService storageProfitlossStatementService;

    final static Logger logger = LoggerFactory.getLogger(StorageProStatementController.class);

    @RequestMapping(value="/getList",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="损溢单申请列表", notes = "查询数据列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name= "sfcxsptgsj", value = "是否查询审批通过的数据(0:是,1:否)", paramType = "query")
    })
    public PageInfo<StorageProfitlossStatement> getList(Integer pageNum, Integer pageSize,HttpServletRequest request, StorageProfitlossStatement storageProfitlossStatement, String sfcxsptgsj) throws Exception {

        List<StorageProfitlossStatement> list = storageProfitlossStatementService.queryByExample(pageNum, pageSize,storageProfitlossStatement, sfcxsptgsj);
        PageInfo<StorageProfitlossStatement> pageInfo = new PageInfo<StorageProfitlossStatement>(list);
        return pageInfo;
    }

    @RequestMapping(value="/queryBasicBusinessData",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="通过仓房货位号查询几个基础数据值", notes = "")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "ch", value = "仓房ID", paramType = "query"),
        @ApiImplicitParam(name = "hwh", value = "货位ID", paramType = "query"),
        @ApiImplicitParam(name = "sfgd", value = "是否归档(0:是,1:否)", paramType = "query")
    })
    public ResponseEntity<Map<String, Object>> queryBasicBusinessData(Integer ch, Integer hwh, String sfgd) {
    	Map<String, Object> map = null;
    	try {
			if (null == ch) {
				return ResponseEntity.failed("ch不能为空");
			} else {
				map = storageProfitlossStatementService.queryBasicBusinessData(ch, hwh, sfgd, null, null);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return ResponseEntity.failed("查询异常");
		}
    	return ResponseEntity.ok(map);
    }

    @RequestMapping(value="/queryLqkAndDcrkData",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="通过仓房货位号和最后一车时间查询这个仓的一条粮情卡数据和倒仓入库的数据", notes = "")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "ch", value = "仓房ID", paramType = "query"),
        @ApiImplicitParam(name = "hwh", value = "货位ID", paramType = "query"),
        @ApiImplicitParam(name = "zhycsj", value = "最后一车时间", paramType = "query")
    })
    public ResponseEntity<Map<String, Object>> queryLqkAndDcrkData(Integer ch, Integer hwh, String zhycsj) {
    	Map<String, Object> map = null;
    	try {
			if (null == ch || StringUtils.isBlank(zhycsj)) {
				return ResponseEntity.failed("ch和zhycsj不能为空");
			} else {
				map = storageProfitlossStatementService.queryLqkAndDcrkData(ch, hwh, zhycsj);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return ResponseEntity.failed("查询异常");
		}
    	return ResponseEntity.ok(map);
    }

    @RequestMapping(value = "/jsRksfzzData", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "入库水分杂质计算记录查询", notes = "入库水分计算查询，支持分页")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "ch", value = "仓房ID", paramType = "query"),
        @ApiImplicitParam(name = "hwh", value = "货位ID", paramType = "query"),
        @ApiImplicitParam(name = "dycrksj", value = "第一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "zhycRksj", value = "最后一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "lspz", value = "粮食品种", paramType = "query"),
        @ApiImplicitParam(name = "orgId", value = "粮库ID", paramType = "query"),
        @ApiImplicitParam(name = "cknf", value = "出库年份", paramType = "query")
    })
    public PageInfo<Map<String, Object>> jsRksfzzData(Integer pageNum, Integer pageSize, 
    		String ch,String hwh,String dycrksj,String zhycRksj, String lspz, String orgId, Integer cknf) {

    	PageInfo<Map<String, Object>> pageInfo = null;
		try {
			if (pageNum != null && pageSize != null) {
	            PageHelper.startPage(pageNum, pageSize);
	        }
			List<Map<String, Object>> list = storageProfitlossStatementService.jsRksfzzData(ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf);
			pageInfo = new PageInfo<Map<String, Object>>(list);
		} catch (Exception e) {
			e.printStackTrace();
		}
        return pageInfo;
    }
    
    @RequestMapping(value = "/jsRksfzzDataSpt", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "入库水分杂质计算记录查询(市平台用)", notes = "支持分页")
    @ApiImplicitParams({
    	@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "spt_hwid", value = "市平台的货位ID", paramType = "query"),
        @ApiImplicitParam(name = "dycrksj", value = "第一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "zhycRksj", value = "最后一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "lspzdm", value = "粮食品种国标码", paramType = "query")
    })
    public Map<String, Object> jsRksfzzDataSpt(Integer pageNum, Integer pageSize, 
    		String spt_hwid, String dycrksj, String zhycRksj, String lspzdm) {

    	Map<String, Object> resultMap = new HashMap<>();
		try {
			if (pageNum != null && pageSize != null) {
	            PageHelper.startPage(pageNum, pageSize);
	        }
			resultMap = storageProfitlossStatementService.jsRksfzzDataSpt(pageNum, pageSize, spt_hwid, dycrksj, zhycRksj, lspzdm);
		} catch (Exception e) {
			e.printStackTrace();
		}
        return resultMap;
    }

    @RequestMapping(value = "/jsRksfzzDataCount", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "入库水分杂质计算记录总数查询", notes = "入库水分计算查询，支持分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "ch", value = "仓房ID", paramType = "query"),
        @ApiImplicitParam(name = "hwh", value = "货位ID", paramType = "query"),
        @ApiImplicitParam(name = "dycrksj", value = "第一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "zhycRksj", value = "最后一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "lspz", value = "粮食品种", paramType = "query"),
        @ApiImplicitParam(name = "orgId", value = "粮库ID", paramType = "query"),
        @ApiImplicitParam(name = "cknf", value = "出库年份", paramType = "query")
    })
    public Map<String, Object> jsRksfzzDataCount(String ch,String hwh,String dycrksj,String zhycRksj, String lspz, String orgId, Integer cknf) {

    	Map<String, Object> result = null;
		try {
			result = storageProfitlossStatementService.jsRksfzzDataCount(ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf);
		} catch (Exception e) {
			e.printStackTrace();
		}
        return result;
    }
    
    @RequestMapping(value="/exportJsRksfzzDataData", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="条件导出入库记录数据", notes = "条件查询和列表一致")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ch", value = "仓房ID", paramType = "query"),
        @ApiImplicitParam(name = "hwh", value = "货位ID", paramType = "query"),
        @ApiImplicitParam(name = "dycrksj", value = "第一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "zhycRksj", value = "最后一车入库时间", paramType = "query"),
        @ApiImplicitParam(name = "lspz", value = "粮食品种", paramType = "query"),
        @ApiImplicitParam(name = "orgId", value = "粮库ID", paramType = "query"),
        @ApiImplicitParam(name = "cknf", value = "出库年份", paramType = "query"),
        @ApiImplicitParam(name = "ywlx", value = "业务类型(0:水分,1:杂质)", paramType = "query")
	})
	public void exportJsRksfzzDataData(HttpServletResponse response,HttpServletRequest request, 
			String ch,String hwh,String dycrksj,String zhycRksj, String lspz, String orgId, Integer cknf, String ywlx){
		try {
			HSSFWorkbook workbook = storageProfitlossStatementService.exportJsRksfzzDataData(ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf, ywlx);
			String fileName = "入库记录导出";
			// 获取输出流
			OutputStream os = response.getOutputStream();
			// 重置输出流
			response.reset();
			// 设定输出文件头
			response.setHeader("Content-disposition", "attachment; filename=" + new String(fileName.getBytes("GB2312"), "8859_1") + ".xlsx");
			//name属性设置和跨服访问有关
			response.setHeader("Access-Control-Allow-Origin", "*");
			// 定义输出内容类型
			response.setContentType("application/vnd.ms-excel;charset=gb2312");
			workbook.write(os);
			//关闭流
			os.close();
			return;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}

    @RequestMapping(value = "/getCksl", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "出库数量计算记录查询", notes = "出库数量计算查询，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ch", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "hwh", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name="rksj", value="封仓时间", paramType="query"),
            @ApiImplicitParam(name = "orgId", value = "粮库ID", paramType = "query")
    })
    public List<Map<String, Object>> getCksl(Integer ch, Integer hwh, String rksj, Integer orgId) {
        Map<String, Object> map = new HashMap<String, Object>();
        if (ch != null && !"".equals(ch)) {
            map.put("ch", ch);
        }
        if (hwh != null && !"".equals(hwh)) {
            map.put("hwh", hwh);
        }
        if (rksj != null && !"".equals(rksj)) {
            map.put("rksj", rksj);
        }
        if (orgId != null && !"".equals(orgId)) {
            map.put("orgId", orgId);
        }
        map.put("dqsj", new Date());
        List<Map<String, Object>> list = storageProfitlossStatementService.getCksl(map);
        return list;
    }

    @RequestMapping(value="/save",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="保存数据", notes = "保存")
    @SuppressWarnings("all")
    @ApiImplicitParams({
            @ApiImplicitParam(name="profitlossStatementJson", value="主表数据", paramType="form"),
            @ApiImplicitParam(name="profitlossDcjlJson", value="子表数据", paramType="form"),
            @ApiImplicitParam(name="orgId", value="当前登录人的组织id", paramType="form")
    })
    public Map<String, Object> save(String profitlossStatementJson, String profitlossDcjlJson, Integer orgId,HttpServletRequest request) {

        Map<String, Object> returnMap = new HashMap<String, Object>();
        try {
        	storageProfitlossStatementService.saveOrUpdate(profitlossStatementJson, profitlossDcjlJson, orgId);
            returnMap.put("status", "success");
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            returnMap.put("status", "error");
            returnMap.put("msg", "保存失败！");
            return returnMap;
        }
        return returnMap;
    }

	@RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除", notes = "根据id删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "id", paramType = "query")
	})
	public Map<String, Object> remove(String id) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		if (id != null) {
			int a = storageProfitlossStatementService.remove(id);
			if(a>0){
				modelMap.put("status", "success");
			}else{
				modelMap.put("status", "error");
			}
		}
		
		return modelMap;
	}

	@RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据id查找", notes = "根据id查找")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "id", paramType = "query")
	})
	public Map<String, Object> edit(String id) {
		Map<String, Object> map = null;
		try {
			if (id != null) {
				map = storageProfitlossStatementService.findById(id);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return map;
	}

    @RequestMapping(value="/submit",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    @ApiOperation(value="损溢单提交", notes = "损溢单提交")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "profitlossStatement", value = "业务主体", paramType = "query")
    })
    public Map<String, Object> submit(String profitlossStatementJson,Integer userId,String userName,String orgName,HttpServletRequest request) {
    	ObjectMapper mapper = new ObjectMapper();
    	Map<String, Object> returnMap = new HashMap<String, Object>();
        StorageProfitlossStatement storageProfitlossStatement;
		try {
			storageProfitlossStatement = (StorageProfitlossStatement)mapper.readValue(profitlossStatementJson, StorageProfitlossStatement.class);
			returnMap = storageProfitlossStatementService.submit(storageProfitlossStatement,userId,userName,orgName);

		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}

        return returnMap;
    }

    
}