package com.chinaitop.depot.profitlossStatement.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.profitlossStatement.mapper.DataStatisticsMapper;
import com.chinaitop.depot.profitlossStatement.model.DataStatistics;
import com.chinaitop.depot.profitlossStatement.service.DataStatisticsService;
import com.chinaitop.depot.utils.HSSFWorkBookUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@Service
public class DataStatisticsServiceImpl implements DataStatisticsService {

	@Resource
	private DataStatisticsMapper dataStatisticsMapper;

	/**
	 * 获取合计行数据
	 * @return
	 */
	private DataStatistics getHjDate(List<DataStatistics> listall) {

		DataStatistics hj_obj = new DataStatistics();

		//入库数量合计
		BigDecimal rksl_hj = listall.stream().map(DataStatistics::getRksl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setRksl(rksl_hj);

		//出库数量合计
		BigDecimal cksl_hj = listall.stream().map(DataStatistics::getCksl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setCksl(cksl_hj);

		//损失数量合计
		BigDecimal sssl_hj = listall.stream().map(DataStatistics::getSssl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setSssl(sssl_hj);

		//损耗数量合计
		BigDecimal shsl_hj = listall.stream().map(DataStatistics::getShsl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setShsl(shsl_hj);

		//溢余数量合计
		BigDecimal yysl_hj = listall.stream().map(DataStatistics::getYysl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setYysl(yysl_hj);

		//超耗数量合计
		BigDecimal chsl_hj = listall.stream().map(DataStatistics::getChsl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setChsl(chsl_hj);
		
		//水杂减量合计
		BigDecimal szjlhj_hj = listall.stream().map(DataStatistics::getSzjlhj).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setSzjlhj(szjlhj_hj);
		
		//保管自然损耗定额数量合计
		BigDecimal bgzrshdesl_hj = listall.stream().map(DataStatistics::getBgzrshdesl).reduce(BigDecimal.ZERO, BigDecimal::add);
		hj_obj.setBgzrshdesl(bgzrshdesl_hj);

		//合计损失率(四射五入保留两位小数):损失数量 / 入库数量 * 100
		BigDecimal ssl_hj = sssl_hj.divide(rksl_hj, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
		hj_obj.setSsl(ssl_hj);

		//合计损耗率(四射五入保留两位小数):损耗数量 / 入库数量 * 100
		BigDecimal shl_hj = shsl_hj.divide(rksl_hj, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
		hj_obj.setShl(shl_hj);

		return hj_obj;
	}
	@Override
	public Map<String, Object> selectByCflx(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) throws Exception {
		Map<String, Object> map = new HashMap<>();
		PageInfo<DataStatistics> pageInfo = null;
		Map<String, String> paramMap = new HashMap<String, String>();
		if (StringUtils.isNotBlank(orgCode)) {
			paramMap.put("orgCode", orgCode);
		}
		if (StringUtils.isNotBlank(startTime)) {
			startTime = startTime+" 00:00:00";
			paramMap.put("startTime", startTime);
		}
		if (StringUtils.isNotBlank(endTime)) {
			endTime = endTime+" 23:59:59";
			paramMap.put("endTime", endTime);
		}
		List<DataStatistics> listall = dataStatisticsMapper.selectByCflx(paramMap);

		//获取合计行数据
		DataStatistics hj_obj = getHjDate(listall);

		map.put("ht_object", hj_obj);

		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<DataStatistics> list = dataStatisticsMapper.selectByCflx(paramMap);
		pageInfo = new PageInfo<DataStatistics>(list);

		map.put("pageInfo", pageInfo);

		return map;
	}

	@Override
	public Map<String, Object> selectByLspz(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) throws Exception {
		Map<String, Object> map = new HashMap<>();

		PageInfo<DataStatistics> pageInfo = null;

        Map<String, String> paramMap = new HashMap<String, String>();
        if (StringUtils.isNotBlank(orgCode)) {
			paramMap.put("orgCode", orgCode);
		}
        if (StringUtils.isNotBlank(startTime)) {
			startTime = startTime+" 00:00:00";
			paramMap.put("startTime", startTime);
		}
		if (StringUtils.isNotBlank(endTime)) {
			endTime = endTime+" 23:59:59";
			paramMap.put("endTime", endTime);
		}
		List<DataStatistics> listall = dataStatisticsMapper.selectByLspz(paramMap);

		//获取合计行数据
		DataStatistics hj_obj = getHjDate(listall);

		map.put("ht_object", hj_obj);
		
		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<DataStatistics> list = dataStatisticsMapper.selectByLspz(paramMap);
		pageInfo = new PageInfo<DataStatistics>(list);

		map.put("pageInfo", pageInfo);

		return map;
	}

	@Override
	public Map<String, Object> selectByLsxz(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) throws Exception {
		Map<String, Object> map = new HashMap<>();

		PageInfo<DataStatistics> pageInfo = null;

		Map<String, String> paramMap = new HashMap<String, String>();
		if (StringUtils.isNotBlank(orgCode)) {
			paramMap.put("orgCode", orgCode);
		}
		if (StringUtils.isNotBlank(startTime)) {
			startTime = startTime+" 00:00:00";
			paramMap.put("startTime", startTime);
		}
		if (StringUtils.isNotBlank(endTime)) {
			endTime = endTime+" 23:59:59";
			paramMap.put("endTime", endTime);
		}

		List<DataStatistics> listall = dataStatisticsMapper.selectByLsxz(paramMap);

		//获取合计行数据
		DataStatistics hj_obj = getHjDate(listall);

		map.put("ht_object", hj_obj);

		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}

		List<DataStatistics> list = dataStatisticsMapper.selectByLsxz(paramMap);
		pageInfo = new PageInfo<DataStatistics>(list);

		map.put("pageInfo", pageInfo);

		return map;
	}

	@Override
	public Map<String, Object> selectByKdmc(Integer pageNum, Integer pageSize, String startTime, String endTime, String orgCode) throws Exception {
		Map<String, Object> map = new HashMap<>();

		PageInfo<DataStatistics> pageInfo = null;

		Map<String, String> paramMap = new HashMap<String, String>();
		if (StringUtils.isNotBlank(orgCode)) {
			paramMap.put("orgCode", orgCode);
		}
		if (StringUtils.isNotBlank(startTime)) {
			startTime = startTime+" 00:00:00";
			paramMap.put("startTime", startTime);
		}
		if (StringUtils.isNotBlank(endTime)) {
			endTime = endTime+" 23:59:59";
			paramMap.put("endTime", endTime);
		}

		List<DataStatistics> listall = dataStatisticsMapper.selectByKdmc(paramMap);

		//获取合计行数据
		DataStatistics hj_obj = getHjDate(listall);

		map.put("ht_object", hj_obj);

		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<DataStatistics> list = dataStatisticsMapper.selectByKdmc(paramMap);
		pageInfo = new PageInfo<DataStatistics>(list);

		map.put("pageInfo", pageInfo);

		return map;
	}

	@Override
	public HSSFWorkbook exportData(String startTime, String endTime, String orgCode) throws Exception {
		HSSFWorkbook wb = new HSSFWorkbook();

		CellRangeAddress cellRange = new CellRangeAddress(0, 0, 0, 11); // A1到J1的区域

		// 创建"仓房类型统计"工作表
		HSSFSheet sheetCflx = wb.createSheet("仓房类型统计");
		sheetCflx.addMergedRegion(cellRange);
		setSheetData(wb, sheetCflx, "cflx", startTime, endTime, orgCode);

		HSSFSheet sheetLspz = wb.createSheet("粮食品种统计");
		sheetLspz.addMergedRegion(cellRange);
		setSheetData(wb, sheetLspz, "lspz", startTime, endTime, orgCode);

		HSSFSheet sheetLsxz = wb.createSheet("粮食性质统计");
		sheetLsxz.addMergedRegion(cellRange);
		setSheetData(wb, sheetLsxz, "lsxz", startTime, endTime, orgCode);

		HSSFSheet sheetKdmc = wb.createSheet("库点统计");
		sheetKdmc.addMergedRegion(cellRange);
		setSheetData(wb, sheetKdmc, "kdmc", startTime, endTime, orgCode);

		return wb;
	}

	@SuppressWarnings("unchecked")
	public void setSheetData(HSSFWorkbook wb, HSSFSheet sheet, String sheetNum, String startTime, String endTime, String orgCode) throws Exception {
		HSSFCellStyle headStyle = HSSFWorkBookUtils.headStyle(wb);
		HSSFCellStyle contentStyle = HSSFWorkBookUtils.contentStyle(wb);
		
		sheet.setColumnWidth(2, 20*256);
		sheet.setColumnWidth(3, 20*256);
		sheet.setColumnWidth(4, 20*256);
		sheet.setColumnWidth(5, 20*256);
		sheet.setColumnWidth(6, 20*256);
		sheet.setColumnWidth(7, 20*256);
		sheet.setColumnWidth(8, 20*256);
		sheet.setColumnWidth(9, 20*256);
		sheet.setColumnWidth(10, 20*256);
		sheet.setColumnWidth(11, 20*256);

		//第一行：标题行
		HSSFRow sheetRow = sheet.createRow(0);
		HSSFCell cell = sheetRow.createCell(0);
		String tilleName = "";
		List<DataStatistics> list = null;
		Map<String, Object> map = null;
		DataStatistics ds = null;
		String cell2Name = ""; //第二行第二列的名称
		if ("cflx".equals(sheetNum)) {
			tilleName = "按照仓房类型统计分析汇总表";
			cell2Name = "仓房类型";
			sheet.setColumnWidth(1, 18*256);
			map = selectByCflx(null, null, startTime, endTime, orgCode);
			ds = (DataStatistics) map.get("ht_object");
			list = ((PageInfo<DataStatistics>)map.get("pageInfo")).getList();
		} else if ("lspz".equals(sheetNum)) {
			tilleName = "按照粮食品种统计分析汇总表";
			cell2Name = "粮食品种";
			sheet.setColumnWidth(1, 20*256);
			map = selectByLspz(null, null, startTime, endTime, orgCode);
			ds = (DataStatistics) map.get("ht_object");
			list = ((PageInfo<DataStatistics>)map.get("pageInfo")).getList();
		} else if ("lsxz".equals(sheetNum)) {
			tilleName = "按照粮食性质统计分析汇总表";
			cell2Name = "粮食性质";
			sheet.setColumnWidth(1, 40*256);
			map = selectByLsxz(null, null, startTime, endTime, orgCode);
			ds = (DataStatistics) map.get("ht_object");
			list = ((PageInfo<DataStatistics>)map.get("pageInfo")).getList();
		} else if ("kdmc".equals(sheetNum)) {
			tilleName = "按照库点统计分析汇总表";
			cell2Name = "库点名称";
			sheet.setColumnWidth(1, 45*256);
			map = selectByKdmc(null, null, startTime, endTime, orgCode);
			ds = (DataStatistics) map.get("ht_object");
			list = ((PageInfo<DataStatistics>)map.get("pageInfo")).getList();
		}

		cell.setCellValue(tilleName);
		cell.setCellStyle(headStyle);

		//第二行
		HSSFRow sheetRow1 = sheet.createRow(1);//列名所在行
		HSSFCell rowCell1 = sheetRow1.createCell(0);
		rowCell1.setCellValue("序号");
		rowCell1.setCellStyle(contentStyle);
		HSSFCell rowCell2 = sheetRow1.createCell(1);
		//第二行第二列的名称
		rowCell2.setCellValue(cell2Name);
		rowCell2.setCellStyle(contentStyle);
		HSSFCell rowCell3 = sheetRow1.createCell(2);
		rowCell3.setCellValue("入库数量(kg)");
		rowCell3.setCellStyle(contentStyle);
		HSSFCell rowCell4 = sheetRow1.createCell(3);
		rowCell4.setCellValue("出库数量(kg)");
		rowCell4.setCellStyle(contentStyle);
		HSSFCell rowCell5 = sheetRow1.createCell(4);
		rowCell5.setCellValue("损失数量(kg)");
		rowCell5.setCellStyle(contentStyle);
		HSSFCell rowCell6 = sheetRow1.createCell(5);
		rowCell6.setCellValue("损失率(%)");
		rowCell6.setCellStyle(contentStyle);
		HSSFCell rowCell7 = sheetRow1.createCell(6);
		rowCell7.setCellValue("损耗数量(kg)");
		rowCell7.setCellStyle(contentStyle);
		HSSFCell rowCell8 = sheetRow1.createCell(7);
		rowCell8.setCellValue("损耗率(%)");
		rowCell8.setCellStyle(contentStyle);
		HSSFCell rowCell9 = sheetRow1.createCell(8);
		rowCell9.setCellValue("溢余数量(kg)");
		rowCell9.setCellStyle(contentStyle);
		HSSFCell rowCell10 = sheetRow1.createCell(9);
		rowCell10.setCellValue("超耗数量(kg)");
		rowCell10.setCellStyle(contentStyle);
		HSSFCell rowCell11 = sheetRow1.createCell(10);
		rowCell11.setCellValue("水杂减量合计(kg)");
		rowCell11.setCellStyle(contentStyle);
		HSSFCell rowCell12 = sheetRow1.createCell(11);
		rowCell12.setCellValue("保管自然损耗定额(kg)");
		rowCell12.setCellStyle(contentStyle);

		//第三行：合计行
		HSSFRow sheetRow2 = sheet.createRow(2);//列名所在行
		HSSFCell row2Cell1 = sheetRow2.createCell(0);
		row2Cell1.setCellValue("——");
		row2Cell1.setCellStyle(contentStyle);
		HSSFCell row2Cell2 = sheetRow2.createCell(1);
		//第二行第二列的名称
		row2Cell2.setCellValue("合计");
		row2Cell2.setCellStyle(headStyle);
		HSSFCell row2Cell3 = sheetRow2.createCell(2);
		row2Cell3.setCellValue(ds.getRksl()+"");
		row2Cell3.setCellStyle(contentStyle);
		HSSFCell row2Cell4 = sheetRow2.createCell(3);
		row2Cell4.setCellValue(ds.getCksl()+"");
		row2Cell4.setCellStyle(contentStyle);
		HSSFCell row2Cell5 = sheetRow2.createCell(4);
		row2Cell5.setCellValue(ds.getSssl()+"");
		row2Cell5.setCellStyle(contentStyle);
		HSSFCell row2Cell6 = sheetRow2.createCell(5);
		row2Cell6.setCellValue(ds.getSsl()+"");
		row2Cell6.setCellStyle(contentStyle);
		HSSFCell row2Cell7 = sheetRow2.createCell(6);
		row2Cell7.setCellValue(ds.getShsl()+"");
		row2Cell7.setCellStyle(contentStyle);
		HSSFCell row2Cell8 = sheetRow2.createCell(7);
		row2Cell8.setCellValue(ds.getShl()+"");
		row2Cell8.setCellStyle(contentStyle);
		HSSFCell row2Cell9 = sheetRow2.createCell(8);
		row2Cell9.setCellValue(ds.getYysl()+"");
		row2Cell9.setCellStyle(contentStyle);
		HSSFCell row2Cell10 = sheetRow2.createCell(9);
		row2Cell10.setCellValue(ds.getChsl()+"");
		row2Cell10.setCellStyle(contentStyle);
		HSSFCell row2Cell11 = sheetRow2.createCell(10);
		row2Cell11.setCellValue(ds.getSzjlhj()+"");
		row2Cell11.setCellStyle(contentStyle);
		HSSFCell row2Cell12 = sheetRow2.createCell(11);
		row2Cell12.setCellValue(ds.getBgzrshdesl()+"");
		row2Cell12.setCellStyle(contentStyle);

		Integer xh=1;
		int rowNum = 2;
		HSSFRow rowData = null;
		DataStatistics data = null;
		for (int i = 0; i < list.size(); i++) {
			rowNum = rowNum + 1;
			data = list.get(i);
			rowData = sheet.createRow(rowNum);
			HSSFCell rowCellData1 = rowData.createCell(0);
			rowCellData1.setCellValue(xh);
			rowCellData1.setCellStyle(contentStyle);
			HSSFCell rowCellData2 = rowData.createCell(1);
			if ("cflx".equals(sheetNum)) {
				rowCellData2.setCellValue(data.getCflx());
			} else if ("lspz".equals(sheetNum)) {
				rowCellData2.setCellValue(data.getLspz());
			} else if ("lsxz".equals(sheetNum)) {
				rowCellData2.setCellValue(data.getLsxz());
			} else if ("kdmc".equals(sheetNum)) {
				rowCellData2.setCellValue(data.getKdmc());
			}
			rowCellData2.setCellStyle(contentStyle);
			HSSFCell rowCellData3 = rowData.createCell(2);
			rowCellData3.setCellValue(data.getRksl() == null ? "":data.getRksl().toString());
			rowCellData3.setCellStyle(contentStyle);
			HSSFCell rowCellData4 = rowData.createCell(3);
			rowCellData4.setCellValue(data.getCksl() == null ? "":data.getCksl().toString());
			rowCellData4.setCellStyle(contentStyle);
			HSSFCell rowCellData5 = rowData.createCell(4);
			rowCellData5.setCellValue(data.getSssl() == null ? "":data.getSssl().toString());
			rowCellData5.setCellStyle(contentStyle);
			HSSFCell rowCellData6 = rowData.createCell(5);
			rowCellData6.setCellValue(data.getSsl() == null ? "":data.getSsl().toString());
			rowCellData6.setCellStyle(contentStyle);
			HSSFCell rowCellData7 = rowData.createCell(6);
			rowCellData7.setCellValue(data.getShsl() == null ? "":data.getShsl().toString());
			rowCellData7.setCellStyle(contentStyle);
			HSSFCell rowCellData8 = rowData.createCell(7);
			rowCellData8.setCellValue(data.getShl() == null ? "":data.getShl().toString());
			rowCellData8.setCellStyle(contentStyle);
			HSSFCell rowCellData9 = rowData.createCell(8);
			rowCellData9.setCellValue(data.getYysl() == null ? "":data.getYysl().toString());
			rowCellData9.setCellStyle(contentStyle);
			HSSFCell rowCellData10 = rowData.createCell(9);
			rowCellData10.setCellValue(data.getChsl() == null ? "":data.getChsl().toString());
			rowCellData10.setCellStyle(contentStyle);
			HSSFCell rowCellData11 = rowData.createCell(10);
			rowCellData11.setCellValue(data.getSzjlhj() == null ? "":data.getSzjlhj().toString());
			rowCellData11.setCellStyle(contentStyle);
			HSSFCell rowCellData12 = rowData.createCell(11);
			rowCellData12.setCellValue(data.getBgzrshdesl() == null ? "":data.getChsl().toString());
			rowCellData12.setCellStyle(contentStyle);
			++xh;
		}
	}
}
