package com.chinaitop.depot.profitlossStatement.service.impl;


import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.feign.BasicFeignClient;
import com.chinaitop.depot.profitlossStatement.mapper.BusinessApprovalMapper;
import com.chinaitop.depot.profitlossStatement.mapper.StorageProfitlossDcjlMapper;
import com.chinaitop.depot.profitlossStatement.mapper.StorageProfitlossStatementMapper;
import com.chinaitop.depot.profitlossStatement.mapper.StorageRksfzzqrdMapper;
import com.chinaitop.depot.profitlossStatement.mapper.SydBusinessMapper;
import com.chinaitop.depot.profitlossStatement.model.BusinessApproval;
import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossDcjl;
import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossDcjlExample;
import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossStatement;
import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossStatementExample;
import com.chinaitop.depot.profitlossStatement.model.StorageProfitlossStatementExample.Criteria;
import com.chinaitop.depot.profitlossStatement.model.StorageRksfzzqrd;
import com.chinaitop.depot.profitlossStatement.model.StorageRksfzzqrdExample;
import com.chinaitop.depot.profitlossStatement.service.StorageProfitlossStatementService;
import com.chinaitop.depot.system.service.SystemOutsideDataService;
import com.chinaitop.depot.utils.AuditType;
import com.chinaitop.depot.utils.HSSFWorkBookUtils;
import com.chinaitop.depot.utils.HistoryList;
import com.chinaitop.depot.utils.PageHelperUtils;
import com.chinaitop.depot.utils.ParameterUtil;
import com.chinaitop.depot.utils.createKC;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;


@Service
public class StorageProfitlossStatementServiceImpl implements StorageProfitlossStatementService {
	
	final static Logger logger = LoggerFactory.getLogger(StorageProfitlossStatementServiceImpl.class);

	@Resource
	private StorageProfitlossStatementMapper storageProfitlossStatementMapper;

	@Resource
	private StorageProfitlossDcjlMapper storageProfitlossDcjlMapper;

	@Resource
	private SydBusinessMapper sydBusinessMapper;

	@Resource
	private BusinessApprovalMapper businessApprovalMapper;

	@Resource
	private StorageRksfzzqrdMapper storageRksfzzqrdMapper;
	
	@Resource
	private BasicFeignClient feignBasicService;

	@Resource
    private SystemOutsideDataService systemOutsideDataService;

	// 获取ip地址
    @Value("${crk-webservice}")
    private String url;

	@Override
	public List<StorageProfitlossStatement> queryByExample(Integer pageNum, Integer pageSize,
			StorageProfitlossStatement storageProfitlossStatement, String sfcxsptgsj) {

		StorageProfitlossStatementExample example = new StorageProfitlossStatementExample();
        Criteria createCriteria = example.createCriteria();
        
        // 仓房/油罐.
        if (StringUtils.isNotBlank(storageProfitlossStatement.getCforyg())){
            createCriteria.andCforygEqualTo(storageProfitlossStatement.getCforyg());
        }
        //货位
        if(storageProfitlossStatement.getHwid() != null){
        	createCriteria.andHwidEqualTo(storageProfitlossStatement.getHwid());
        }
        
        // 审批状态.
        if (storageProfitlossStatement.getAuditState() != null){
        	if(storageProfitlossStatement.getAuditState().equals(15)){//五级审批时的状态 全部为审批中
        		createCriteria.andAuditStateGreaterThanOrEqualTo(1);
        		createCriteria.andAuditStateLessThanOrEqualTo(5);
        	}else{
                createCriteria.andAuditStateEqualTo(storageProfitlossStatement.getAuditState());
        	}
        }
        
        if(storageProfitlossStatement.getOrgId() != null){
        	createCriteria.andOrgIdEqualTo(storageProfitlossStatement.getOrgId());
        }
        //是否查询审批通过的数据(0:是,1:否)
        if ("1".equals(sfcxsptgsj)) {
        	createCriteria.andAuditStateNotEqualTo(6);
        }
        
        example.setOrderByClause("create_time desc");
      
        if (null != pageNum && null != pageSize) {
        	PageHelper.startPage(pageNum, pageSize);
        }
        
        List<StorageProfitlossStatement> selectByExample = storageProfitlossStatementMapper.selectByExample(example);
        return selectByExample;
	}

	@Override
	public List<Map<String, Object>> jsRksfzzData(String ch, String hwh, String dycrksj, String zhycRksj, String lspz,
			String orgId, Integer cknf) throws Exception {

		List<Map<String, Object>> list = rkData(ch, hwh, dycrksj, zhycRksj, lspz);

		if (null == list || list.size() == 0) {
			//出库时间是属于2024年之前，并且没在出入库业务表查到记录，那么聪入库水分、杂质确认单里面取
			if (null != cknf && cknf < 2024 && StringUtils.isNotBlank(hwh)) {
				StorageRksfzzqrdExample example = new StorageRksfzzqrdExample();
				StorageRksfzzqrdExample.Criteria criteria = example.createCriteria();
				criteria.andCfidEqualTo(Integer.parseInt(ch));
				criteria.andHwidEqualTo(Integer.parseInt(hwh));
				List<StorageRksfzzqrd> rksfzzqrd_list = storageRksfzzqrdMapper.selectByExample(example);
				if (null != rksfzzqrd_list && rksfzzqrd_list.size() > 0) {
					StorageRksfzzqrd storageRksfzzqrd = rksfzzqrd_list.get(0);
					Map<String, Object> resultMap = new HashMap<>();
			        resultMap.put("cphm", "");
			        resultMap.put("srsl", storageRksfzzqrd.getRksl());
			        resultMap.put("sfjyz", storageRksfzzqrd.getRksf());
			        resultMap.put("zzjyz", storageRksfzzqrd.getRkzz());
			        resultMap.put("rcsj", ParameterUtil.datetime2string(storageRksfzzqrd.getJksj()));
			        BigDecimal sf = storageRksfzzqrd.getRksf()
			        		.divide(new BigDecimal("100"), 5, BigDecimal.ROUND_DOWN)
			        		.multiply(storageRksfzzqrd.getRksl()).setScale(1, RoundingMode.HALF_UP);
					resultMap.put("sf", sf);
					BigDecimal zz = storageRksfzzqrd.getRkzz()
			        		.divide(new BigDecimal("100"), 5, BigDecimal.ROUND_DOWN)
			        		.multiply(storageRksfzzqrd.getRksl()).setScale(1, RoundingMode.HALF_UP);
					resultMap.put("zz", zz);
					list = new ArrayList<>();
					list.add(resultMap);
				}
			}
		}
		return list;
	}

	/**
	 * 只查询正规入库记录
	 * @param ch
	 * @param hwh
	 * @param dycrksj
	 * @param zhycRksj
	 * @param lspz
	 * @return
	 * @throws Exception
	 */
	public List<Map<String, Object>> rkData(String ch, String hwh, String dycrksj, String zhycRksj, String lspz) throws Exception {
		Map<String, String> paramMap = queryCondition(ch, hwh, dycrksj, zhycRksj, lspz);
		List<Map<String, Object>> list = null;
		if (StringUtils.isBlank(hwh)) {
			list = storageProfitlossStatementMapper.jsRksfzzTankData(paramMap);
		} else {
			list = storageProfitlossStatementMapper.jsRksfzzData(paramMap);
		}
		return list;
	}

	public Map<String, String> queryCondition(String ch, String hwh, String dycrksj, String zhycRksj, String lspz) {
		Map<String, String> paramMap = new HashMap<>();
		paramMap.put("ch", ch);
		paramMap.put("hwh", hwh);
		paramMap.put("startTime", dycrksj);
		paramMap.put("endTime", zhycRksj);
        //大豆2777  sfbm=2809 zzbm=2813
        //小麦3164  sfbm=2783 zzbm=2784
        //玉米3165  sfbm=2791 zzbm=2792
        //稻谷3166  sfbm=2799 zzbm=2802
		if (lspz.equals("2777")) {
			paramMap.put("sfbm", "2809");
			paramMap.put("zzbm", "2874");
		} else if (lspz.equals("3164")) {
			paramMap.put("sfbm", "2783");
			paramMap.put("zzbm", "2784");
		} else if (lspz.equals("3165")) {
			paramMap.put("sfbm", "2791");
			paramMap.put("zzbm", "2792");
		} else if (lspz.equals("3166")) {
			paramMap.put("sfbm", "2799");
			paramMap.put("zzbm", "2802");
		}
		return paramMap;
	}

	@Override
	public Map<String, Object> jsRksfzzDataCount(String ch, String hwh, String dycrksj, String zhycRksj, String lspz,
			String orgId, Integer cknf) throws Exception {

		Map<String, Object> result = new HashMap<>();

		//水分总和
		BigDecimal sf_count = new BigDecimal(0);

		//水分检验值总和
		BigDecimal sfjyz_count = new BigDecimal(0);

		//杂质总和
		BigDecimal zz_count = new BigDecimal(0);

		//杂质总和
		BigDecimal zzjyz_count = new BigDecimal(0);

		//收入数量总和
		BigDecimal srsl_count = new BigDecimal(0);

		//拿到所有符合条件的数据
		Map<String, String> paramMap = queryCondition(ch, hwh, dycrksj, zhycRksj, lspz);
		Map<String, Object> map = storageRksfzzqrdMapper.queryRksfAndRkzz(paramMap);

		if (null != map) {

			srsl_count = new BigDecimal(map.get("rkzl").toString());

			sf_count = new BigDecimal(map.get("sf").toString());

			zz_count = new BigDecimal(map.get("zz").toString());

			sfjyz_count = sf_count.divide(srsl_count, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);

			zzjyz_count = zz_count.divide(srsl_count, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);
		} else {
			if (null != cknf && cknf < 2024) {
				StorageRksfzzqrdExample example = new StorageRksfzzqrdExample();
				StorageRksfzzqrdExample.Criteria criteria = example.createCriteria();
				criteria.andCfidEqualTo(Integer.parseInt(ch));
				criteria.andHwidEqualTo(Integer.parseInt(hwh));
				List<StorageRksfzzqrd> rksfzzqrd_list = storageRksfzzqrdMapper.selectByExample(example);
				if (null != rksfzzqrd_list && rksfzzqrd_list.size() > 0) {
					StorageRksfzzqrd storageRksfzzqrd = rksfzzqrd_list.get(0);

					srsl_count = storageRksfzzqrd.getRksl();

					sf_count = storageRksfzzqrd.getYwsfzl();
					if (null == sf_count) {
						sf_count = new BigDecimal(0);
					}

					zz_count = storageRksfzzqrd.getYwzzzl();
					if (null == zz_count) {
						zz_count = new BigDecimal(0);
					}

					sfjyz_count = sf_count.divide(srsl_count, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);

					zzjyz_count = zz_count.divide(srsl_count, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);
				}
			}
		}

		result.put("srsl_count", srsl_count);
		result.put("sf_count", sf_count);
		result.put("zz_count", zz_count);
		result.put("sfjyz_count", sfjyz_count);
		result.put("zzjyz_count", zzjyz_count);

		return result;
	}

	@Override
	public List<Map<String, Object>> getCksl(Map<String, Object> map) {
		return storageProfitlossStatementMapper.getCksl(map);
	}

	@Override
	public int add(StorageProfitlossStatement storageProfitlossStatement) {
		return storageProfitlossStatementMapper.insertSelective(storageProfitlossStatement);
	}

	@Override
	public int update(StorageProfitlossStatement storageProfitlossStatement) {
		return storageProfitlossStatementMapper.updateByPrimaryKeySelective(storageProfitlossStatement);
	}

	@Override
	public int remove(String id) {
		return storageProfitlossStatementMapper.deleteByPrimaryKey(id);
	}

	@Override
	public Map<String, Object> findById(String id) throws Exception{
		Map<String, Object> map = new HashMap<>();
		StorageProfitlossStatement storageProfitlossStatement = findByIdProfitlossStatement(id);
		map.put("storageProfitlossStatement", storageProfitlossStatement);

		StorageProfitlossDcjlExample example = new StorageProfitlossDcjlExample();
		StorageProfitlossDcjlExample.Criteria criteria = example.createCriteria();
		criteria.andZidEqualTo(storageProfitlossStatement.getId());
		example.setOrderByClause("cjsj");
		List<StorageProfitlossDcjl> list = storageProfitlossDcjlMapper.selectByExample(example);
		map.put("dclist", list);
		return map;
	}

	@Override
	@Transactional(readOnly = false)
	public Map<String, Object> submit(StorageProfitlossStatement profitlossStatement,Integer userId,String userName,String orgName) {
		
	        Map<String, Object> returnMap = new HashMap<String, Object>();
	        
	        
	        Integer auditState = profitlossStatement.getAuditState();
	        if (auditState!=null &&
	                 (!auditState.equals(AuditType.UNSUBMIT.getAuditCode()) && 
	                 !auditState.equals(AuditType.REJECT.getAuditCode()))) {
	            returnMap.put("success", "error");
	            returnMap.put("msg", "您已经提交该数据，无法再次提交！");
	            return returnMap;
	        }
	        
	        /**
	         * 审批信息数据
	         */
	        BusinessApproval businessApproval = new BusinessApproval();
			businessApproval.setOperateTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));//操作时间
	        businessApproval.setOperator(userId+"");//已选择的审批人id
	        businessApproval.setOperatorName(userName);//已选择的审批人姓名
	        businessApproval.setCreateTime(new Date()); //创建时间
	        businessApproval.setFromDepartment(orgName); //部门 orgName
	    	businessApproval.setProjectName("损溢单审批");//名称
	        businessApproval.setTaskType("spill");//任务类型
	    	
	    	businessApproval.setApplyTime(profitlossStatement.getCreateTime());//申请时间
	        businessApproval.setTaskTypeName(profitlossStatement.getSydh());//任务类型名
	        businessApproval.setIsHide(0);//
	        businessApproval.setContent("同意");//审批意见
	        businessApproval.setTaskId(profitlossStatement.getId());
	        businessApproval.setTaskName("员工提交");//任务名称
	        
	        // 设置审批状态为，已经提交.等待第一级审批
	        profitlossStatement.setAuditState(AuditType.AUDIT1.getAuditCode());
	        // 修改.
	        int a = storageProfitlossStatementMapper.updateByPrimaryKeySelective(profitlossStatement);
	        if(a>0){
	        	 //增加审批表数据
	            int b = businessApprovalMapper.insertSelective(businessApproval);
	            if(b>0){
	            	returnMap.put("success", "success");
	            }else{
	            	returnMap.put("success", "error");
		            returnMap.put("msg", "提交失败！");
	            }
	        	
	        }else{
	        	returnMap.put("success", "error");
	            returnMap.put("msg", "提交失败！");
	        }
	        
	        return returnMap;
	    
	}

	@Override
	public List<Map<String, String>> getUserRole(Integer userId) {
		return storageProfitlossStatementMapper.getUserRole(userId);
	}

	@Override
	public List<HistoryList> getAudit(Map<String, Object> modelMap) {
		return businessApprovalMapper.getAudit(modelMap);
	}

	@Override
	public Map<String, Object> audit(StorageProfitlossStatement storageProfitlossStatement, Integer userId, String realName,
			String auditResult, String content,String orgName) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
        
		BusinessApproval businessApproval = new BusinessApproval();
		businessApproval.setOperateTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));//操作时间
        businessApproval.setOperator(userId+"");//已选择的审批人id
        businessApproval.setOperatorName(realName);//已选择的审批人姓名
        businessApproval.setCreateTime(new Date()); //创建时间
        businessApproval.setFromDepartment(orgName); //部门 orgName
    	businessApproval.setProjectName("损溢单审批");//名称
        businessApproval.setTaskType("spill");//任务类型
    	
    	businessApproval.setApplyTime(storageProfitlossStatement.getCreateTime());//申请时间
        businessApproval.setTaskTypeName(storageProfitlossStatement.getSydh());//任务类型名
        businessApproval.setIsHide(0);//
        if(content != null && content != ""){
        	 businessApproval.setContent(content);//审批意见
        }else{
        	 businessApproval.setContent("同意");//审批意见
        }
       
        businessApproval.setTaskId(storageProfitlossStatement.getId());
        
        String result = "";
        //1通过2驳回3拒绝
        if ("1".equals(auditResult)) {
        	result = "同意";
        	
        	if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT1.getAuditCode())){//仓储部审批
            	//更改该条数据
            	storageProfitlossStatement.setCcshr(userId);
            	storageProfitlossStatement.setAuditState(AuditType.AUDIT2.getAuditCode());
            	
            	//审批表数据的新增
                businessApproval.setTaskName(AuditType.AUDIT1.getAuditName());//任务名称

            }else{
            	
            	if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT2.getAuditCode())){//质检审批
            		storageProfitlossStatement.setAuditState(AuditType.AUDIT3.getAuditCode());
            		storageProfitlossStatement.setZjshr(userId);
            		businessApproval.setTaskName(AuditType.AUDIT2.getAuditName());//任务名称
            		
            	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT3.getAuditCode())){//统计审批
            		storageProfitlossStatement.setAuditState(AuditType.AUDIT4.getAuditCode());
            		storageProfitlossStatement.setTjshr(userId);
            		businessApproval.setTaskName(AuditType.AUDIT3.getAuditName());//任务名称
            		
            	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT4.getAuditCode())){//会计审批
            		storageProfitlossStatement.setAuditState(AuditType.AUDIT5.getAuditCode());
            		storageProfitlossStatement.setKjshr(userId);
            		businessApproval.setTaskName(AuditType.AUDIT4.getAuditName());//任务名称
            	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT5.getAuditCode())){//库领导审批
            		storageProfitlossStatement.setAuditState(AuditType.AGREE.getAuditCode());//审批完毕
            		storageProfitlossStatement.setLdshr(userId);
            		businessApproval.setTaskName(AuditType.AUDIT5.getAuditName());//任务名称
            	}
            	
            }
        	
        } else if ("2".equals(auditResult) || "3".equals(auditResult)) {
        	

        	/**
        	 * 审批表的数据修改
        	 */
        	if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT1.getAuditCode())){//仓储部审批
            	//审批表数据的新增
                businessApproval.setTaskName(AuditType.AUDIT1.getAuditName());//任务名称

            }else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT2.getAuditCode())){//质检审批
            	
        		businessApproval.setTaskName(AuditType.AUDIT2.getAuditName());//任务名称
        		
        	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT3.getAuditCode())){//统计审批
        		
        		businessApproval.setTaskName(AuditType.AUDIT3.getAuditName());//任务名称
        		
        	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT4.getAuditCode())){//会计审批
        		
        		businessApproval.setTaskName(AuditType.AUDIT4.getAuditName());//任务名称
        		
        	}else if(storageProfitlossStatement.getAuditState().equals(AuditType.AUDIT5.getAuditCode())){//库领导审批
        		
        		businessApproval.setTaskName(AuditType.AUDIT5.getAuditName());//任务名称
        	}
        	
        	if("2".equals(auditResult)){
        		
        		result = "驳回";
        		/**
            	 * 驳回到申请后驳回状态 可以重新提交
            	 * 主表之前的审批人不做变动 后面的提交审批 会覆盖之前的值
            	 */
            	storageProfitlossStatement.setAuditState(AuditType.REJECT.getAuditCode());
            	
        	}else if("3".equals(auditResult)){
        		result = "拒绝";
            	/**
            	 * 拒绝 则整条数据结束
            	 */
            	storageProfitlossStatement.setAuditState(AuditType.REFUSE.getAuditCode());
        	}
        } 
        
        businessApproval.setResult(result);//审批结果
        
       //修改本条数据
    	int  a = storageProfitlossStatementMapper.updateByPrimaryKeySelective(storageProfitlossStatement);
    	if (a>0){
    		 //增加审批表数据
             int b = businessApprovalMapper.insertSelective(businessApproval);
             if(b>0){
            	 modelMap.put("status", "success");
            	 if (storageProfitlossStatement.getAuditState() == 6) {//把数据给出入库应急端推送过去
            		 String url_ip = storageProfitlossStatementMapper.getCrkYjdAddress(storageProfitlossStatement.getOrgId().toString());
            		 if (StringUtils.isNotBlank(url_ip)) {
            			 //拼接完整URL
            			 StringBuffer sbf_url = new StringBuffer(100);
            			 sbf_url.append(url_ip).append(url);

            			 StorageProfitlossStatement profitlossStatemnt = storageProfitlossStatementMapper.selectByPrimaryKey(storageProfitlossStatement.getId());
            			 if (StringUtils.isNotBlank(profitlossStatemnt.getRksj().toString())) {
            				 profitlossStatemnt.setRksj_str(ParameterUtil.datetime2string(profitlossStatemnt.getRksj()));
            			 }
            			 if (StringUtils.isNotBlank(profitlossStatemnt.getCksj().toString())) {
            				 profitlossStatemnt.setCksj_str(ParameterUtil.datetime2string(profitlossStatemnt.getCksj()));
            			 }

            			 //获取保管员
            			 String keepName = storageProfitlossStatementMapper.getkeepName(storageProfitlossStatement.getCforyg());
            			 if (StringUtils.isNotBlank(keepName)) {
            				 profitlossStatemnt.setKeepName(keepName);
            			 }
            			 
            			 modelMap.put("data", profitlossStatemnt);

            			 //组装参数数据
            			 JSONObject obj = new JSONObject();
            			 obj.put("data", profitlossStatemnt);
            			 obj.put("message", "success");
            			 obj.put("retCode", 200);

            			 Object[] parameters = new Object[1];
            			 String param = "["+JSON.toJSONString(obj)+"]";
            			 parameters[0] = param;
            			 

            			 logger.info("损益单推送地址是："+sbf_url.toString());
            			 logger.info("损益单推送参数是："+parameters);
            			 String str = createKC.createKC("createSyd", sbf_url.toString(), parameters);
            			 systemOutsideDataService.addlogger("syd_dycrk", param, str);
            			 logger.info("损益单推送结果是："+str);

            		 }

            	 }
            }else{
            	 modelMap.put("status", "error");
            	 modelMap.put("msg", "流程数据新增不成功");
            }
    	}else{
    		modelMap.put("status", "error");
       	    modelMap.put("msg", "原数据修改不成功");
    	}
      
		return modelMap;
	}

	@Override
	public HSSFWorkbook exportJsRksfzzDataData(String ch, String hwh, String dycrksj, String zhycRksj, String lspz,
			String orgId, Integer cknf, String ywlx) throws Exception {
		HSSFWorkbook wb = new HSSFWorkbook();

		CellRangeAddress cellRange = new CellRangeAddress(0, 0, 0, 5); // A1到F1的区域

		// 创建"入库信息"sheet工作表
		HSSFSheet sheetRkxx = wb.createSheet("入库信息");//sheet页名称
		sheetRkxx.addMergedRegion(cellRange);
		setSheetData(wb, sheetRkxx, ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf, ywlx);

		return wb;
	}

	public void setSheetData(HSSFWorkbook wb, HSSFSheet sheet, String ch, String hwh, String dycrksj, String zhycRksj, String lspz,
			String orgId, Integer cknf, String ywlx) throws Exception {
		HSSFCellStyle headStyle = HSSFWorkBookUtils.headStyle(wb);
		HSSFCellStyle contentStyle = HSSFWorkBookUtils.contentStyle(wb);
		
		//每一列的样式宽度
		sheet.setColumnWidth(1, 25*256);
		sheet.setColumnWidth(2, 25*256);
		sheet.setColumnWidth(3, 25*256);
		sheet.setColumnWidth(4, 25*256);
		sheet.setColumnWidth(5, 25*256);
		sheet.setColumnWidth(6, 25*256);

		//创建sheet页的标题行(第1行)
		HSSFRow sheetRow0 = sheet.createRow(0);
		HSSFCell cell = sheetRow0.createCell(0);
		//页签名称
		cell.setCellValue("入库记录");
		cell.setCellStyle(headStyle);

		//获取数据列表
		List<Map<String, Object>> list = jsRksfzzData(ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf);

		//获取合计数据
		Map<String, Object> count_data = jsRksfzzDataCount(ch, hwh, dycrksj, zhycRksj, lspz, orgId, cknf);

		//第二行
		HSSFRow sheetRow1 = sheet.createRow(1);//列名所在行
		//第二行第一列
		HSSFCell rowCell1 = sheetRow1.createCell(0);
		rowCell1.setCellValue("序号");
		rowCell1.setCellStyle(contentStyle);
		//第二行第二列
		HSSFCell rowCell2 = sheetRow1.createCell(1);
		rowCell2.setCellValue("汽车牌号");
		rowCell2.setCellStyle(contentStyle);
		//第二行第三列
		HSSFCell rowCell3 = sheetRow1.createCell(2);
		rowCell3.setCellValue("数量(kg)");
		rowCell3.setCellStyle(contentStyle);
		//第二行第四列
		HSSFCell rowCell4 = sheetRow1.createCell(3);
		if ("0".equals(ywlx)) {
			rowCell4.setCellValue("水分(%)");
		} else {
			rowCell4.setCellValue("杂质(%)");
		}
		rowCell4.setCellStyle(contentStyle);
		//第二行第五列
		HSSFCell rowCell5 = sheetRow1.createCell(4);
		if ("0".equals(ywlx)) {
			rowCell5.setCellValue("数量*水分");
		} else {
			rowCell5.setCellValue("数量*杂质");
		}
		rowCell5.setCellStyle(contentStyle);
		//第二行第六列
		HSSFCell rowCell6 = sheetRow1.createCell(5);
		rowCell6.setCellValue("入仓时间");
		rowCell6.setCellStyle(contentStyle);

		//第三行
		HSSFRow sheetRow2 = sheet.createRow(2);//合计所在行
		//第三行第一列
		HSSFCell row3Cell1 = sheetRow2.createCell(0);
		row3Cell1.setCellValue("合计");
		row3Cell1.setCellStyle(contentStyle);
		//第三行第二列
		HSSFCell row3Cell2 = sheetRow2.createCell(1);
		row3Cell2.setCellValue("————");
		row3Cell2.setCellStyle(contentStyle);
		//第三行第三列
		HSSFCell row3Cell3 = sheetRow2.createCell(2);
		row3Cell3.setCellValue(count_data.get("srsl_count").toString());
		row3Cell3.setCellStyle(contentStyle);
		//第三行第四列
		HSSFCell row3Cell4 = sheetRow2.createCell(3);
		if ("0".equals(ywlx)) {
			row3Cell4.setCellValue(count_data.get("sfjyz_count").toString());
		} else {
			row3Cell4.setCellValue(count_data.get("zzjyz_count").toString());
		}
		row3Cell4.setCellStyle(contentStyle);
		//第三行第五列
		HSSFCell row3Cell5 = sheetRow2.createCell(4);
		if ("0".equals(ywlx)) {
			row3Cell5.setCellValue(count_data.get("sf_count").toString());
		} else {
			row3Cell5.setCellValue(count_data.get("zz_count").toString());
		}
		row3Cell5.setCellStyle(contentStyle);
		//第三行第六列
		HSSFCell row3Cell6 = sheetRow2.createCell(5);
		row3Cell6.setCellValue("————");
		row3Cell6.setCellStyle(contentStyle);

		Integer xh=1;
		int rowNum = 2;
		Map<String, Object> map = null;
		HSSFRow rowData = null;
		if (null != list && list.size() > 0) {
			for (int i = 0; i < list.size(); i++) {
				rowNum = rowNum + 1;
				rowData = sheet.createRow(rowNum);
				
				//第一列
				map = list.get(i);
				HSSFCell rowCellData1 = rowData.createCell(0);
				rowCellData1.setCellValue(xh);
				rowCellData1.setCellStyle(contentStyle);
				//第二列
				HSSFCell rowCellData2 = rowData.createCell(1);
				rowCellData2.setCellValue(map.get("cphm").toString());
				rowCellData2.setCellStyle(contentStyle);
				//第三列
				HSSFCell rowCellData3 = rowData.createCell(2);
				rowCellData3.setCellValue(map.get("srsl").toString());
				rowCellData3.setCellStyle(contentStyle);
				//第四列
				HSSFCell rowCellData4 = rowData.createCell(3);
				if ("0".equals(ywlx)) {
					rowCellData4.setCellValue(map.get("sfjyz").toString());
				} else {
					rowCellData4.setCellValue(map.get("zzjyz").toString());
				}
				rowCellData4.setCellStyle(contentStyle);
				//第五列
				HSSFCell rowCellData5 = rowData.createCell(4);
				if ("0".equals(ywlx)) {
					rowCellData5.setCellValue(map.get("sf").toString());
				} else {
					rowCellData5.setCellValue(map.get("zz").toString());
				}
				rowCellData5.setCellStyle(contentStyle);
				//第六列
				HSSFCell rowCellData6 = rowData.createCell(5);
				rowCellData6.setCellValue(map.get("rcsj").toString());
				rowCellData6.setCellStyle(contentStyle);
				
				++xh;
			}
		}
	}

	@Override
	public void test(String id) throws Exception {

		StorageProfitlossStatement syd = storageProfitlossStatementMapper.selectByPrimaryKey(id);

		String url_ip = storageProfitlossStatementMapper.getCrkYjdAddress(syd.getOrgId().toString());

		if (StringUtils.isNotBlank(url_ip)) {
			//拼接完整URL
		    StringBuffer sbf_url = new StringBuffer(100);
		    sbf_url.append(url_ip).append(url);
		    
		    if (StringUtils.isNotBlank(syd.getRksj().toString())) {
		    	syd.setRksj_str(ParameterUtil.datetime2string(syd.getRksj()));
		    }
		    if (StringUtils.isNotBlank(syd.getCksj().toString())) {
		    	syd.setCksj_str(ParameterUtil.datetime2string(syd.getCksj()));
		    }
		    
		    //获取保管员
		    String keepName = storageProfitlossStatementMapper.getkeepName(syd.getCforyg());
		    if (StringUtils.isNotBlank(keepName)) {
		    	syd.setKeepName(keepName);
		    }
		    //组装参数数据
		    JSONObject obj = new JSONObject();
		    obj.put("data", syd);
		    obj.put("message", "success");
		    obj.put("retCode", 200);

		    Object[] parameters = new Object[1];
		    String parame = "["+JSON.toJSONString(obj)+"]";
		    parameters[0] = parame;
		 

		    logger.info("损益单推送地址是："+sbf_url.toString());
		    logger.info("损益单推送参数是："+parame);
			String str = createKC.createKC("createSyd", sbf_url.toString(), parameters);
			logger.info("损益单推送结果是："+str);
		}

	}

	@Override
	public void saveOrUpdate(String profitlossStatementJson, String profitlossDcjlJson, Integer orgId) throws Exception {
		// JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        StorageProfitlossStatement storageProfitlossStatement = (StorageProfitlossStatement)mapper.readValue(profitlossStatementJson, StorageProfitlossStatement.class);

        Date date = new Date();
        if (storageProfitlossStatement.getId() == null) {
        	storageProfitlossStatement.setId(UUID.randomUUID().toString().replace("-", ""));
        	storageProfitlossStatement.setAuditState(0);//待提交
        	storageProfitlossStatement.setOrgId(orgId);//单位id
        	storageProfitlossStatement.setCreateTime(date);
        	storageProfitlossStatement.setUpdateTime(date);
        	storageProfitlossStatement.setSydh(createSydh(storageProfitlossStatement));
            add(storageProfitlossStatement);//新增主表数据
        } else {
        	storageProfitlossStatement.setUpdateTime(date);
        	update(storageProfitlossStatement);//修改主表数据
        }

        //保存子表数据
        String dclist = profitlossDcjlJson.replace("[", "").replace("]", "");
        if (StringUtils.isNotBlank(dclist)) {
        	addOrUpdateDcjl(profitlossDcjlJson, orgId, storageProfitlossStatement.getId());
        }
	}

	private String createSydh(StorageProfitlossStatement storageProfitlossStatement) {
		String sydh = "";
		StringBuffer sbf = new StringBuffer(20);
		sbf.append(storageProfitlossStatement.getHwdm()).append(ParameterUtil.date2string2(storageProfitlossStatement.getCreateTime())).append("001");
		sydh = sbf.toString();
		return sydh;
	}

	public void addOrUpdateDcjl(String profitlossDcjlJson, Integer orgId, String zid) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		List<StorageProfitlossDcjl> dcjlLisst = mapper.readValue(profitlossDcjlJson, new TypeReference<List<StorageProfitlossDcjl>>(){});
		if (null != dcjlLisst && dcjlLisst.size() > 0) {
			Date date = new Date();
			for (StorageProfitlossDcjl storageProfitlossDcjl : dcjlLisst) {
				if (StringUtils.isBlank(storageProfitlossDcjl.getId())) {
					storageProfitlossDcjl.setId(UUID.randomUUID().toString().replace("-", ""));
					storageProfitlossDcjl.setOrgId(orgId);
					storageProfitlossDcjl.setZid(zid);
					storageProfitlossDcjl.setCjsj(date);
					storageProfitlossDcjl.setUpdatetime(date);
					storageProfitlossDcjlMapper.insert(storageProfitlossDcjl);
				} else {
					storageProfitlossDcjl.setUpdatetime(date);
					storageProfitlossDcjlMapper.updateByPrimaryKey(storageProfitlossDcjl);
				}
			}
		}
	}

	@Override
	public StorageProfitlossStatement findByIdProfitlossStatement(String id) throws Exception {
		StorageProfitlossStatement storageProfitlossDcjl = storageProfitlossStatementMapper.selectByPrimaryKey(id);
		return storageProfitlossDcjl;
	}

	@Override
	public Map<String, Object> queryBasicBusinessData(Integer ch, Integer hwh, String sfgd, String startTime, String endTime) throws Exception {

		Map<String, Object> param = new HashMap<>();
		if (null != ch) {
			param.put("ch", ch);
		}
		if (null != hwh) {
			param.put("hwh", hwh);
		}
		if (StringUtils.isNotBlank(sfgd)) {
			param.put("sfgd", sfgd);
		}
		if (StringUtils.isNotBlank(startTime)) {
			param.put("startTime", startTime);
		}
		if (StringUtils.isNotBlank(endTime)) {
			param.put("endTime", endTime);
		}

		Map<String, Object> result = sydBusinessMapper.queryBasicBusinessData(param);

		return result;
	}

	@Override
	public Map<String, Object> queryLqkAndDcrkData(Integer ch, Integer hwh, String zhycsj) throws Exception {

		//返回的数据
		Map<String, Object> result_map = new HashMap<>();

		Map<String, Object> dr_param = new HashMap<>();
		dr_param.put("drch", ch);
		dr_param.put("drhwh", hwh);
		dr_param.put("startTime", zhycsj);

		//查有没有倒入作业
		List<Map<String, Object>> dcrkList = sydBusinessMapper.queryDcrkData(dr_param);

		//如果有，就把倒仓的数据拿出来，粮情卡数据取倒入前最后的一条粮情卡和对应的质检数据
		if (null != dcrkList && dcrkList.size() > 0) {
			for (int i = 0; i < dcrkList.size(); i++) {

				Map<String, Object> dc_map = dcrkList.get(i);

				//获取导出仓房的入仓水分(%)、入仓杂质(%)、封仓自检水分(%)、封仓自检杂质(%)
				dc_map = getDccfSfAndZz(dc_map);
			}

			//获取当前仓房倒入前的封仓自检水分(%)、封仓自检杂质(%)
			String endTime = dcrkList.get(0).get("sqsj").toString();//第一条倒入数据作业申请时间
			Map<String, Object> this_cf_map = queryBasicBusinessData(ch, hwh, null, null, endTime);
			if (null != hwh) {
				result_map.put("fczjsf", this_cf_map.get("fczjsf").toString());//当前仓房在倒入前的封仓自检水分
				result_map.put("fczjzz", this_cf_map.get("fczjzz").toString());//当前仓房在倒入前的封仓自检杂质
			} else {
				result_map.put("fczjsf", 0);//当前仓房在倒入前的封仓自检水分
				result_map.put("fczjzz", 0);//当前仓房在倒入前的封仓自检杂质
			}
			

		} else {//如果没有， 取当前仓的最后一条粮情卡的数据和对应的质检数据，
			Map<String, Object> this_cf_map = queryBasicBusinessData(ch, hwh, null, null, null);
			if (null != hwh) {
				result_map.put("fczjsf", this_cf_map.get("fczjsf").toString());//当前仓房的封仓自检水分
				result_map.put("fczjzz", this_cf_map.get("fczjzz").toString());//当前仓房的封仓自检杂质
			} else {
				result_map.put("fczjsf", 0);//当前仓房的封仓自检水分
				result_map.put("fczjzz", 0);//当前仓房的封仓自检杂质
			}
			
		}
		
		//获取出库数量、出库时间
		result_map = getCkslAndCksj(ch, hwh, zhycsj, result_map);

		//获取当前仓房的出仓水分(%)、出仓杂质(%)
		if (null != hwh) {//粮食
			Map<String, Object> param = new HashMap<>();
			param.put("ch", ch);
			param.put("hwh", hwh);
			param.put("startTime", zhycsj);
			Map<String, Object> ck_map = sydBusinessMapper.queryCksfAndZz(param);
			result_map.put("ccsf", ck_map.get("ccsf").toString());//当前仓房的出仓水分
			result_map.put("cczz", ck_map.get("cczz").toString());//当前仓房的出仓杂质
		} else {//油罐
			result_map.put("ccsf", 0);//当前仓房的出仓水分
			result_map.put("cczz", 0);//当前仓房的出仓杂质
		}

		//获取倒仓出库数量
		result_map = getDcCksl(ch, hwh, zhycsj, result_map);

		result_map.put("dclist", dcrkList);

		return result_map;
	}

	/**
	 * 获取导出仓房的入仓水分(%)、入仓杂质(%)、封仓自检水分(%)、封仓自检杂质(%)
	 * @param dc_map
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unused")
	private Map<String, Object> getDccfSfAndZz(Map<String, Object> dc_map) throws Exception {
		//1、先找到这个仓倒出之前的最后一个粮情卡
		String endTime = dc_map.get("sqsj").toString();//倒出作业申请时间
		Integer dcch = Integer.parseInt(dc_map.get("dcch").toString());
		Integer dchwh = Integer.parseInt(dc_map.get("dchwh")+"");

		Map<String, Object> dclqk = queryBasicBusinessData(dcch, dchwh, null, null, endTime);
		if (null != dchwh) {
			dc_map.put("fczjsf", dclqk.get("fczjsf").toString());//倒出仓房的封仓自检水分
			dc_map.put("fczjzz", dclqk.get("fczjzz").toString());//倒出仓房的封仓自检杂质
		} else {
			dc_map.put("fczjsf", 0);//倒出仓房的封仓自检水分
			dc_map.put("fczjzz", 0);//倒出仓房的封仓自检杂质
		}

		//2、从中获取第一车和最后一车时间
		String dycsj = dclqk.get("dycsj").toString();
		String zhycsj = dclqk.get("zhycsj").toString();
		String lspz = dclqk.get("lspz").toString();
		List<Map<String, Object>> list = jsRksfzzData(dcch.toString(), dchwh+"", dycsj, zhycsj, lspz, null, Integer.parseInt(zhycsj.substring(0, 4)));
		if (null != list && list.size() > 0) {
			BigDecimal zsl = new BigDecimal("0");
			BigDecimal zsf = new BigDecimal("0");
			BigDecimal zzz = new BigDecimal("0");
			for (Map<String, Object> map : list) {
				zsl = zsl.add(new BigDecimal(map.get("srsl").toString()));
				if (null != dchwh) {
					zsf = zsf.add(new BigDecimal(map.get("sf").toString()));
					zzz = zzz.add(new BigDecimal(map.get("zz").toString()));
				}
			}

			if (null != dchwh) {
				//倒出仓房的入库水分检验值
				BigDecimal sfjyz_count = zsf.divide(zsl, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);
				//倒出仓房的入库杂质检验值
				BigDecimal zzjyz_count = zzz.divide(zsl, 5, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(1, RoundingMode.HALF_UP);
				dc_map.put("rcsf", sfjyz_count);
				dc_map.put("rczz", zzjyz_count);
			} else {
				dc_map.put("rcsf", 0);
				dc_map.put("rczz", 0);
			}

		}
		
		return dc_map;
	}

	private Map<String, Object> getCkslAndCksj(Integer ch, Integer hwh, String zhycsj, Map<String, Object> map) throws Exception {
		
		Map<String, Object> ck_map = new HashMap<String, Object>();
    	ck_map.put("ch", ch);
    	ck_map.put("hwh", hwh);
    	ck_map.put("rksj", zhycsj);
        ck_map.put("dqsj", new Date());
        List<Map<String, Object>> list = getCksl(ck_map);
        if (null != list && list.size() > 0) {
        	map.put("cksj", list.get(0).get("rq").toString());
        	Map<String, Object> obj = null;
        	BigDecimal cksl = new BigDecimal(0);
        	for (int i = 0; i < list.size(); i++) {
        		obj = list.get(i);
        		cksl = cksl.add(new BigDecimal(obj.get("zcsl").toString()));
        	}
        	map.put("cksl", cksl);
        }
        
        return map;
	}
	
	private Map<String, Object> getDcCksl(Integer ch, Integer hwh, String zhycsj, Map<String, Object> map) throws Exception {
		
		Map<String, Object> ck_map = new HashMap<String, Object>();
    	ck_map.put("dcch", ch);
    	ck_map.put("dchwh", hwh);
    	ck_map.put("startTime", zhycsj);
    	List<Map<String, Object>> list = sydBusinessMapper.queryDcrkData(ck_map);
        if (null != list && list.size() > 0) {
        	Map<String, Object> obj = null;
        	BigDecimal dcsl = new BigDecimal(0);
        	for (int i = 0; i < list.size(); i++) {
        		obj = list.get(i);
        		dcsl = dcsl.add(new BigDecimal(obj.get("drlssl").toString()));
        	}
        	map.put("dccksl", dcsl);
        } else {
        	map.put("dccksl", 0);
        }
        
        return map;
	}

	@Override
	public Map<String, Object> jsRksfzzDataSpt(Integer pageNum, Integer pageSize, String spt_hwid, String dycrksj,
			String zhycRksj, String lspzdm) throws Exception {

		Map<String, Object> resultMap = new HashMap<>();
		//校验粮食品种
		String lspz = "";
		if (StringUtils.isNotBlank(lspzdm)) {
			if (lspzdm.equals("2130000") || lspzdm.equals("1410000") || lspzdm.equals("2131000")) {
				lspz = "7144";
				logger.info("根据"+lspzdm+"获取到的品种数据如下：");
				logger.info("原油对应的字典ID是：7144");
			} else {
				Map<String, Object> basicMap = feignBasicService.getIdByNameAndParentId(1061, null, lspzdm);
				lspz = basicMap.get("enumId")+"";
				logger.info("根据"+lspzdm+"获取到的品种数据如下：");
				logger.info(basicMap.toString());
			}
			if (StringUtils.isBlank(lspz)) {
				resultMap.put("msg", "粮食品种代码的值"+lspzdm+"有误或不合理,");
				resultMap.put("status", "error");
				return resultMap;
			}
		} else {
			resultMap.put("msg", "粮食品种代码不能为空,");
			resultMap.put("status", "error");
			return resultMap;
		}

		//校验货位
		String houseId = null;
		String warehouseId = null;
		String orgId = null;
		if (StringUtils.isNotBlank(spt_hwid)) {
			Map<String, Object> basicMap = feignBasicService.getWareOrTankObj(spt_hwid);
			logger.info("根据值"+spt_hwid+"获取到的货位数据如下：");
			logger.info(basicMap+"");
			if (!"0".equals(basicMap.get("houseId").toString()) && !"0".equals(basicMap.get("warehouseId").toString()) && !"0".equals(basicMap.get("orgId").toString())) {
				houseId = basicMap.get("houseId").toString();
				if (spt_hwid.length() > 4) {//说明是粮食仓房的。如果是小于等于4，那就是油罐的数据，货位的值为空
					warehouseId = basicMap.get("warehouseId").toString();
				}
				orgId = basicMap.get("orgId").toString();
			} else {
				resultMap.put("msg", "未查询到值spt_hwid对应的货位数据,");
				resultMap.put("status", "error");
				return resultMap;
			}
		} else {
			resultMap.put("msg", "spt_hwid的值不能为空,");
			resultMap.put("status", "error");
			return resultMap;
		}

		/* 获取List分页数据 start */
		Integer cknf = Integer.parseInt(zhycRksj.substring(0, 4));
		List<Map<String, Object>> list = jsRksfzzData(houseId, warehouseId, dycrksj, zhycRksj, lspz, orgId, cknf);
		ObjectMapper mapper = new ObjectMapper();
        String jsonString = mapper.writeValueAsString(list);
		PageInfo<Object> pageInfo = PageHelperUtils.pageHelper2(jsonString, pageNum, pageSize);
		resultMap.put("pageInfoData", pageInfo);
		/***** 获取List分页数据 end *****/

		/****** 获取合计数据 start ******/
		Map<String, Object> hj_map = jsRksfzzDataCount(houseId, warehouseId, dycrksj, zhycRksj, lspz, orgId, cknf);
		resultMap.put("hj_data", hj_map);
		/****** 获取合计数据 end ******/

		return resultMap;
	}
}
