package com.chinaitop.depot.profitlossStatement.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.profitlossStatement.mapper.StorageRksfzzqrdMapper;
import com.chinaitop.depot.profitlossStatement.model.StorageRksfzzqrd;
import com.chinaitop.depot.profitlossStatement.model.StorageRksfzzqrdExample;
import com.chinaitop.depot.profitlossStatement.service.StorageRksfzzqrdService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;

@Service
public class StorageRksfzzqrdServiceImpl implements StorageRksfzzqrdService {

	@Resource
	private StorageRksfzzqrdMapper storageRksfzzqrdMapper;

	@Override
	public List<StorageRksfzzqrd> findByCondition(Integer pageNum, Integer pageSize, Integer orgId) throws Exception {

		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}

		StorageRksfzzqrdExample example = new StorageRksfzzqrdExample();
		StorageRksfzzqrdExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		example.setOrderByClause("cfid, hwid asc");

		List<StorageRksfzzqrd> list = storageRksfzzqrdMapper.selectByExample(example);

		return list;
	}

	private List<StorageRksfzzqrd> getRksfAndRkzz(List<StorageRksfzzqrd> list) {
		Map<String, String> paramMap = null;
		Map<String, Object> resultMap = null;
		for (StorageRksfzzqrd i : list) {
			//System.out.println(i.getHwid().toString());
			paramMap = new HashMap<>();
			paramMap.put("hwh", i.getHwid().toString());
			paramMap.put("startTime", ParameterUtil.datetime2string(i.getRksj()));
			paramMap.put("endTime", ParameterUtil.datetime2string(i.getRcsj()));
//			大豆2777  sfbm=2809 zzbm=2813
//			小麦3164  sfbm=2783 zzbm=2784
//			玉米3165  sfbm=2791 zzbm=2792
//			稻谷3166  sfbm=2799 zzbm=2802
			if (i.getPz().equals("2777")) {
				paramMap.put("sfbm", "2809");
				paramMap.put("zzbm", "2874");
			} else if (i.getPz().equals("3164")) {
				paramMap.put("sfbm", "2783");
				paramMap.put("zzbm", "2784");
			} else if (i.getPz().equals("3165")) {
				paramMap.put("sfbm", "2791");
				paramMap.put("zzbm", "2792");
			} else if (i.getPz().equals("3166")) {
				paramMap.put("sfbm", "2799");
				paramMap.put("zzbm", "2802");
			}
			resultMap = storageRksfzzqrdMapper.queryRksfAndRkzz(paramMap);

			if (null != resultMap) {
				BigDecimal ywsfzl = new BigDecimal(resultMap.get("sf").toString());
				i.setYwsfzl(ywsfzl);
				//入库水分=逐车的水分之和除以入库总数，在乘以100%，保留1为小数，小数位直接截取，不做四舍五入等其它操作
				BigDecimal rksf = ywsfzl.divide(i.getRksl(), 5, BigDecimal.ROUND_DOWN)
						.multiply(new BigDecimal(100))
						.setScale(1, RoundingMode.HALF_UP);
				i.setRksf(rksf);
				
				BigDecimal ywzzzl = new BigDecimal(resultMap.get("zz").toString());
				i.setYwzzzl(ywzzzl);
				//入库杂质=逐车的杂质之和除以入库总数，在乘以100%，保留1为小数，小数位直接截取，不做四舍五入等其它操作
				BigDecimal rkzz = ywzzzl.divide(i.getRksl(), 5, BigDecimal.ROUND_DOWN)
						.multiply(new BigDecimal(100))
						.setScale(1, RoundingMode.HALF_UP);
				i.setRkzz(rkzz);
//				System.out.println("货位ID："+i.getHwid()+"，入库数量："+i.getRksl()+"，入库水分："+resultMap.get("sf")+"，入库杂质："+resultMap.get("zz"));
			}
			i.setSfqr("1");
			i.setUpdateTime(new Date());
			i.setId(UUID.randomUUID().toString().replace("-", ""));
		}
		return list;
	}

	@Override
	public void save(String orgId, Integer cfid) throws Exception {
		Map<String, Object> paramter_map = new HashMap<>();
		if (StringUtils.isNotBlank(orgId)) {
			paramter_map.put("orgId", orgId);
		}
		if (null != cfid) {
			paramter_map.put("cfid", cfid);
		}
		List<StorageRksfzzqrd> list = storageRksfzzqrdMapper.querySfqrdJcsj(paramter_map);
		//获取每条数据的入库水分和入库杂质
		if (null != list && list.size() > 0) {
			list = getRksfAndRkzz(list);
			StorageRksfzzqrd storageRksfzzqrd = null;
			StorageRksfzzqrd obj = null;
			for (int i = 0; i < list.size(); i++) {
				storageRksfzzqrd = list.get(i);
				StorageRksfzzqrdExample example = new StorageRksfzzqrdExample();
				StorageRksfzzqrdExample.Criteria criteria = example.createCriteria();
				criteria.andHwidEqualTo(storageRksfzzqrd.getHwid());
				List<StorageRksfzzqrd> obj_list = storageRksfzzqrdMapper.selectByExample(example);
				if (null != obj_list && obj_list.size() > 0) {
					obj = obj_list.get(0);
					storageRksfzzqrd.setId(obj.getId());
					if ("1".equals(obj.getSfqr())) {
						storageRksfzzqrdMapper.updateByPrimaryKey(storageRksfzzqrd);
					}
				} else {
					storageRksfzzqrdMapper.insert(storageRksfzzqrd);
				}
			}
		}
	}

	@Override
	public void update(String rksfzzqrd) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		StorageRksfzzqrd obj = mapper.readValue(rksfzzqrd, StorageRksfzzqrd.class);
		obj.setSfqr("0");
		obj.setUpdateTime(new Date());
		storageRksfzzqrdMapper.updateByPrimaryKey(obj);
	}

}
