package com.chinaitop.depot.yylck.controller;

import java.util.List;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.chinaitop.depot.yylck.model.StorageYylckDefault;
import com.chinaitop.depot.yylck.service.StorageYylckDefaultAuditService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@SuppressWarnings("all")
@RequestMapping(value="/yylckDefaultAudit")
@Api(value= "StorageYylckDefaultAuditController", description = "溢余粮出库申请控制类")
@RestController
public class StorageYylckDefaultAuditController {

	final static Logger log = LoggerFactory.getLogger(StorageYylckDefaultAuditController.class);

	@Resource
	private StorageYylckDefaultAuditService storageYylckDefaultAuditService;

	@RequestMapping(value="/getAuditList", method = RequestMethod.GET)
	@ApiOperation(value="溢余粮审批列表", notes = "支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "storageYylckDefault", value = "参数对象", paramType = "query")
	})
	public ResponseEntity<PageInfo<StorageYylckDefault>> getAuditList(Integer pageNum, Integer pageSize, 
			StorageYylckDefault storageYylckDefault) {
		PageInfo<StorageYylckDefault> pageInfo = null;
		try {
			if (null == storageYylckDefault.getDlrUserId() || null == storageYylckDefault.getOrgId()) {
				return ResponseEntity.failed("登录人ID和库ID不能为空!");
			}
			if (pageNum != null && pageSize != null) {
	            PageHelper.startPage(pageNum, pageSize);
	        }
			List<StorageYylckDefault> list = storageYylckDefaultAuditService.queryAuditList(storageYylckDefault);
			pageInfo = new PageInfo<>(list);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(pageInfo);
	}

	@RequestMapping(value="/submitFlow", method = RequestMethod.POST)
	@ApiOperation(value="溢余粮审批保存", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "flowObj", value = "参数对象", paramType = "form")
	})
	public ResponseEntity submitFlow(String flowObj) {
		try {
			storageYylckDefaultAuditService.submitFlow(flowObj);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok();
	}
}
