package com.chinaitop.depot.yylck.controller;

import java.util.List;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.chinaitop.depot.yylck.model.StorageYylckDefault;
import com.chinaitop.depot.yylck.service.StorageYylckDefaultService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@SuppressWarnings("all")
@RequestMapping(value="/yylckDefault")
@Api(value= "StorageYylckDefaultController", description = "溢余粮出库申请控制类")
@RestController
public class StorageYylckDefaultController {

	final static Logger log = LoggerFactory.getLogger(StorageYylckDefaultController.class);

	@Resource
	private StorageYylckDefaultService storageYylckDefaultService;

	@RequestMapping(value="/getList", method = RequestMethod.GET)
	@ApiOperation(value="溢余粮申请列表", notes = "支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "storageYylckDefault", value = "数据对象", paramType = "query")
	})
	public ResponseEntity<PageInfo<StorageYylckDefault>> getList(Integer pageNum, Integer pageSize, StorageYylckDefault storageYylckDefault) {
		PageInfo<StorageYylckDefault> pageInfo = null;
		try {
			if (null == storageYylckDefault.getCjrId()) {
				return ResponseEntity.failed("当前登录用户ID不能为空");
			}
			if (pageNum != null && pageSize != null) {
	            PageHelper.startPage(pageNum, pageSize);
	        }
			List<StorageYylckDefault> list = storageYylckDefaultService.queryList(storageYylckDefault);
			pageInfo = new PageInfo<>(list);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(pageInfo);
	}

	@RequestMapping(value="/saveOrUpdate", method = RequestMethod.POST)
	@ApiOperation(value="溢余粮申请保存", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "yylckJson", value = "数据对象json串", paramType = "form"),
        @ApiImplicitParam(name = "tjrId", value = "提交人ID", paramType = "form"),
        @ApiImplicitParam(name = "fileIds", value = "文件信息", paramType = "form")
	})
	public ResponseEntity saveOrUpdate(String yylckJson, Integer tjrId, String fileIds) {
		try {
			storageYylckDefaultService.saveOrUpdate(yylckJson, tjrId, fileIds);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok();
	}

	@RequestMapping(value="/findByIdObj", method = RequestMethod.GET)
	@ApiOperation(value="溢余粮申请数据详情", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "数据主键ID", paramType = "query")
	})
	public ResponseEntity<StorageYylckDefault> findByIdObj(String id) {
		StorageYylckDefault storageYylckDefault = null;
		try {
			storageYylckDefault = storageYylckDefaultService.findByIdYylckDefaultObj(id);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(storageYylckDefault);
	}
	
	@RequestMapping(value="/deleteByIdObj", method = RequestMethod.POST)
	@ApiOperation(value="删除溢余粮申请数据", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "数据主键ID", paramType = "form")
	})
	public ResponseEntity<StorageYylckDefault> deleteByIdObj(String id) {
		StorageYylckDefault storageYylckDefault = null;
		try {
			storageYylckDefaultService.deleteByIdYylckDefaultObj(id);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(storageYylckDefault);
	}
}
