package com.chinaitop.depot.yylck.controller;

import java.util.List;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.chinaitop.depot.yylck.model.StorageYylckDefault;
import com.chinaitop.depot.yylck.model.StorageYylcktzd;
import com.chinaitop.depot.yylck.service.StorageYylckDefaultService;
import com.chinaitop.depot.yylck.service.StorageYylcktzdService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@SuppressWarnings("all")
@RequestMapping(value="/yylcktzd")
@Api(value= "StorageYylcktzdController", description = "溢余粮出库通知单控制类")
@RestController
public class StorageYylcktzdController {

	final static Logger log = LoggerFactory.getLogger(StorageYylcktzdController.class);

	@Resource
	private StorageYylcktzdService storageYylcktzdService;

	@RequestMapping(value="/getList", method = RequestMethod.GET)
	@ApiOperation(value="溢余粮出库通知单列表", notes = "支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
        @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
        @ApiImplicitParam(name = "storageYylcktzd", value = "数据对象", paramType = "query"),
        @ApiImplicitParam(name = "ckkssj_str", value = "出库开始时间字符串", paramType = "query"),
		@ApiImplicitParam(name = "ckjssj_str", value = "出库完成时间字符串", paramType = "query")
	})
	public ResponseEntity<PageInfo<StorageYylcktzd>> getList(Integer pageNum, Integer pageSize, 
			StorageYylcktzd storageYylcktzd, String ckkssj_str, String ckjssj_str) {
		PageInfo<StorageYylcktzd> pageInfo = null;
		try {
			if (pageNum != null && pageSize != null) {
	            PageHelper.startPage(pageNum, pageSize);
	        }
			List<StorageYylcktzd> list = storageYylcktzdService.getList(storageYylcktzd, ckkssj_str, ckjssj_str);
			pageInfo = new PageInfo<>(list);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(pageInfo);
	}

	@RequestMapping(value="/findByIdObj", method = RequestMethod.GET)
	@ApiOperation(value="溢余粮出库通知单数据详情", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "数据主键ID", paramType = "query")
	})
	public ResponseEntity<StorageYylcktzd> findByIdObj(String id) {
		StorageYylcktzd storageYylcktzd = null;
		try {
			storageYylcktzd = storageYylcktzdService.findById(id);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(storageYylcktzd);
	}

	@RequestMapping(value="/saveOrUpdate", method = RequestMethod.POST)
	@ApiOperation(value="溢余粮出库通知单保存", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "yylcktzdJson", value = "数据对象json串", paramType = "form")
	})
	public ResponseEntity saveOrUpdate(String yylcktzdJson) {
		try {
			storageYylcktzdService.saveOrUpdate(yylcktzdJson);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok();
	}

	@RequestMapping(value="/deleteByIdObj", method = RequestMethod.POST)
	@ApiOperation(value="删除溢余粮出库通知单数据", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "数据主键ID", paramType = "form")
	})
	public ResponseEntity<StorageYylcktzd> deleteByIdObj(String id) {
		StorageYylcktzd storageYylcktzd = null;
		try {
			storageYylcktzdService.deleteObj(id);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(storageYylcktzd);
	}

	@RequestMapping(value="/getYylcktzdNo", method = RequestMethod.GET)
	@ApiOperation(value="获取溢余粮出库通知单编号", notes = "")
	@ApiImplicitParams({
        @ApiImplicitParam(name = "orgId", value = "库ID", paramType = "query")
	})
	public ResponseEntity<String> getYylcktzdNo(Integer orgId) {
		String yylcktzdNo = "";
		try {
			yylcktzdNo = storageYylcktzdService.getYylcktzdNo(orgId);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok(yylcktzdNo);
	}
}
