package com.chinaitop.depot.yylck.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.yylck.mapper.BusinessFlowDetailMapper;
import com.chinaitop.depot.yylck.mapper.StorageYylckDefaultMapper;
import com.chinaitop.depot.yylck.model.BusinessFlowDetail;
import com.chinaitop.depot.yylck.model.BusinessFlowDetailExample;
import com.chinaitop.depot.yylck.model.StorageYylckDefault;
import com.chinaitop.depot.yylck.service.StorageYylckDefaultAuditService;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class StorageYylckDefaultAuditServiceImpl implements StorageYylckDefaultAuditService {

	@Resource
	private StorageYylckDefaultMapper storageYylckDefaultMapper;
	
	@Resource
	private BusinessFlowDetailMapper businessFlowDetailMapper;

	@Override
	public List<StorageYylckDefault> queryAuditList(StorageYylckDefault storageYylckDefault) throws Exception {

		List<StorageYylckDefault> list = storageYylckDefaultMapper.queryAuditList(storageYylckDefault);

		return list;
	}

	@Override
	public void submitFlow(String flowObj) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		BusinessFlowDetail flowDetail = (BusinessFlowDetail)mapper.readValue(flowObj, BusinessFlowDetail.class);
		if (StringUtils.isNotBlank(flowDetail.getId()) && StringUtils.isNotBlank(flowDetail.getDataStatus())) {
			BusinessFlowDetailExample example = new BusinessFlowDetailExample();
			BusinessFlowDetailExample.Criteria criteria = example.createCriteria();
			if ("1".equals(flowDetail.getDataStatus())) {
				flowDetail.setClsj(new Date());
				businessFlowDetailMapper.updateByPrimaryKeySelective(flowDetail);

				//查询当前
				criteria.andZidEqualTo(flowDetail.getZid());
				List<BusinessFlowDetail> list = businessFlowDetailMapper.selectByExample(example);
				//检索当前流程中有没有还未审批通过的数据
				List<BusinessFlowDetail> wcl_list = list.stream().filter(flow -> "0".equals(flow.getDataStatus())).collect(Collectors.toList());
				if (wcl_list.size() == 0) {
					if (list.size() < 5) {//四部审批中，该流转到库领导那里了
						//组装库领导审批流程数据
						Map<String, Integer> role_map = businessFlowDetailMapper.selectYylckAuditRoleIds(list.get(0).getOrgId());
						Integer kld = role_map.get("kld");
						BusinessFlowDetail detail = new BusinessFlowDetail();
						detail.setId(UUID.randomUUID().toString().replace("-", ""));
						detail.setOrgId(list.get(0).getOrgId());
						detail.setFlowTitle("溢余粮出库申请审批流程数据");
						detail.setZid(flowDetail.getZid());
						detail.setTjrId(list.get(0).getTjrId());
						detail.setTjrName(list.get(0).getTjrName());
						detail.setClrRoleId(kld);
						detail.setRemark("这是库领导要审批的数据");
						detail.setXh(5);
						detail.setCjsj(new Date());
						detail.setDataStatus("0");//未处理
						businessFlowDetailMapper.insert(detail);
						
						//业务数据状态设置为库领导审批中
						updateStorageYylckDefaultStatus(flowDetail.getZid(), "2");
					}
                    if (list.size() > 4) {//库领导审批中，该结束了
                    	updateStorageYylckDefaultStatus(flowDetail.getZid(), "3");
					}
				}
			} else {
				//驳回， 要删除所有流程数据，并将主业务数据设置为未提交
				criteria.andZidEqualTo(flowDetail.getZid());
				businessFlowDetailMapper.deleteByExample(example);

				updateStorageYylckDefaultStatus(flowDetail.getZid(), "4");
			}
		}
	}

	/**
	 * 修改溢余粮出库主流程状态值
	 * @param id 主键ID
	 * @param status 数据状态(0:待提交,1:待审批,2:库领导审批中,3:审批通过,4:审批驳回)
	 */
	public void updateStorageYylckDefaultStatus(String id, String status) {
		StorageYylckDefault storageYylckDefault = new StorageYylckDefault();
		storageYylckDefault.setId(id);
		storageYylckDefault.setDataStatus(status);
		storageYylckDefault.setUpdatetime(new Date());
		storageYylckDefaultMapper.updateByPrimaryKeySelective(storageYylckDefault);
	}

}
