package com.chinaitop.depot.yylck.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.feign.BasicFeignClient;
import com.chinaitop.depot.utils.ParameterUtil;
import com.chinaitop.depot.yylck.mapper.BusinessFlowDetailMapper;
import com.chinaitop.depot.yylck.mapper.StorageYylckDefaultMapper;
import com.chinaitop.depot.yylck.model.BusinessFlowDetail;
import com.chinaitop.depot.yylck.model.BusinessFlowDetailExample;
import com.chinaitop.depot.yylck.model.StorageYylckDefault;
import com.chinaitop.depot.yylck.model.StorageYylckDefaultExample;
import com.chinaitop.depot.yylck.service.StorageYylckDefaultService;
import com.fasterxml.jackson.databind.ObjectMapper;

@SuppressWarnings("all")
@Service
public class StorageYylckDefaultServiceImpl implements StorageYylckDefaultService {

	@Resource
	private StorageYylckDefaultMapper storageYylckDefaultMapper;

	@Resource
	private BusinessFlowDetailMapper businessFlowDetailMapper;

	@Resource
    private BasicFeignClient basicFeignClient;

	@Override
	public List<StorageYylckDefault> queryList(StorageYylckDefault storageYylckDefault)
			throws Exception {

		StorageYylckDefaultExample example = new StorageYylckDefaultExample();
		StorageYylckDefaultExample.Criteria criteria = example.createCriteria();

		if (null != storageYylckDefault.getOrgId()) {
			criteria.andOrgIdEqualTo(storageYylckDefault.getOrgId()); //库ID
		}
		if (null != storageYylckDefault.getCh()) {
			criteria.andChEqualTo(storageYylckDefault.getCh());  //仓房ID
		}
		if (null != storageYylckDefault.getHwh()) {
			criteria.andHwhEqualTo(storageYylckDefault.getHwh());  //货位ID
		}
		if (StringUtils.isNotBlank(storageYylckDefault.getDataStatus())) {
			criteria.andDataStatusEqualTo(storageYylckDefault.getDataStatus());
		}
		if (null != storageYylckDefault.getCjrId()) {
			criteria.andCjrIdEqualTo(storageYylckDefault.getCjrId());
		}

		example.setOrderByClause("cjsj desc");

		List<StorageYylckDefault> list = storageYylckDefaultMapper.selectByExample(example);

		return list;
	}

	@Override
	public void saveOrUpdate(String yylckJson, Integer tjrId, String fileIds) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		StorageYylckDefault storageYylckDefault = (StorageYylckDefault)mapper.readValue(yylckJson, StorageYylckDefault.class);
		Date date = new Date();
		int count = 0;
		if (StringUtils.isBlank(storageYylckDefault.getId())) {//新增
			storageYylckDefault.setId(UUID.randomUUID().toString().replace("-", ""));
			storageYylckDefault.setApplyNo(getApplyNO(storageYylckDefault.getOrgId(), date));
			storageYylckDefault.setCjsj(date);
			storageYylckDefault.setUpdatetime(date);
			count = storageYylckDefaultMapper.insert(storageYylckDefault);
		} else {//修改
			storageYylckDefault.setUpdatetime(date);
			count = storageYylckDefaultMapper.updateByPrimaryKey(storageYylckDefault);
		}

		//如果提交的话，要生成待审批的数据
		if ("1".equals(storageYylckDefault.getDataStatus())) {//这是提交状态
			if (count > 0) {
				saveForeDepotFlowData(tjrId, storageYylckDefault);
			}
		}
	}

	//生成单号
	private String getApplyNO(Integer orgId, Date date) {
		
		String applyNo = "";
		
		String day = ParameterUtil.date2string1(date);
		Date startTime = ParameterUtil.string2datetime(day+" 00:00:00");
		Date endTime = ParameterUtil.string2datetime(day+" 23:59:59");
		StorageYylckDefaultExample example = new StorageYylckDefaultExample();
		StorageYylckDefaultExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andCjsjBetween(startTime, endTime);
		example.setOrderByClause("cjsj desc");
		List<StorageYylckDefault> list = storageYylckDefaultMapper.selectByExample(example);
		StringBuffer sbf = new StringBuffer(50);
		if (null != list && list.size() > 0) {
			StorageYylckDefault storageYylckDefault = list.get(0);
			String no = storageYylckDefault.getApplyNo().substring(storageYylckDefault.getApplyNo().length()-3, storageYylckDefault.getApplyNo().length());
			Integer num = Integer.parseInt(no)+1;
			String sxm = String.format("%03d", num);
			sbf.append("YYCKSQ").append(ParameterUtil.date2string2(date)).append(sxm);
		} else {
			sbf.append("YYCKSQ").append(ParameterUtil.date2string2(date)).append("001");
		}
		applyNo = sbf.toString();
		return applyNo;
	}

	@Override
	public StorageYylckDefault findByIdYylckDefaultObj(String id) throws Exception {
		//获取主表数据
		StorageYylckDefault storageYylckDefault = storageYylckDefaultMapper.selectByPrimaryKey(id);
		//获取流程表数据
		BusinessFlowDetailExample example = new BusinessFlowDetailExample();
		BusinessFlowDetailExample.Criteria criteria = example.createCriteria();
		criteria.andZidEqualTo(id);
		example.setOrderByClause("xh asc");
		List<BusinessFlowDetail> list = businessFlowDetailMapper.selectByExample(example);
		storageYylckDefault.setAuditList(list);
		return storageYylckDefault;
	}

	@Override
	public void deleteByIdYylckDefaultObj(String id) throws Exception {

		//删除主表数据
		storageYylckDefaultMapper.deleteByPrimaryKey(id);

		//删除流程表数据
		BusinessFlowDetailExample example = new BusinessFlowDetailExample();
		BusinessFlowDetailExample.Criteria criteria = example.createCriteria();
		criteria.andZidEqualTo(id);
		businessFlowDetailMapper.deleteByExample(example);
	}

	/**
	 * 组装溢余粮出库申请的流程数据
	 * 流程设计思路：
	 * 第一：先用五个角色的名称去查当前库有没有这五个角色
	 * 第二：只要有一个角色没查到，那就不能让数据提交成功
	 * 第三：如果五个角色都有，那么就生成除库领导之外的那四个部门的未审批流程数据
	 * @param tjrId
	 * @param storageYylckDefault
	 * @throws Exception
	 */
	public void saveForeDepotFlowData(Integer tjrId, StorageYylckDefault storageYylckDefault) throws Exception {
		//获取当前库的五部门的角色ID
		Map<String, Integer> role_map = businessFlowDetailMapper.selectYylckAuditRoleIds(storageYylckDefault.getOrgId());
		Integer ccbjl = role_map.get("ccbjl");
		Integer zjbjl = role_map.get("zjbjl");
		Integer tjbjl = role_map.get("tjbjl");
		Integer kj = role_map.get("kj");
		Integer kld = role_map.get("kld");
		if (null == ccbjl || null == zjbjl || null == tjbjl || null == kj || null == kld) {
			//如果这五个部门角色有一个是空的，则不能让数据提交成功
			storageYylckDefault.setDataStatus("0");//待提交
			storageYylckDefaultMapper.updateByPrimaryKey(storageYylckDefault);
		} else {
			//组装并四个部门的未处理的流程数据
			BusinessFlowDetail detail = null;
			for (int i = 1; i < 5; i++) {
				detail = new BusinessFlowDetail();
				detail.setId(UUID.randomUUID().toString().replace("-", ""));
				detail.setOrgId(storageYylckDefault.getOrgId());
				detail.setFlowTitle("溢余粮出库申请审批流程数据");
				detail.setZid(storageYylckDefault.getId());
				detail.setTjrId(tjrId);
				detail.setTjrName(storageYylckDefault.getCjr());
				if (i == 1) {
					detail.setClrRoleId(ccbjl);
					detail.setRemark("这是仓储部经理要审批的数据");
				} else if (i == 2) {
					detail.setClrRoleId(zjbjl);
					detail.setRemark("这是质检部经理要审批的数据");
				} else if (i == 3) {
					detail.setClrRoleId(tjbjl);
					detail.setRemark("这是统计部经理要审批的数据");
				} else if (i == 4) {
					detail.setClrRoleId(kj);
					detail.setRemark("这是会计要审批的数据");
				}
				detail.setXh(i);
				detail.setCjsj(new Date());
				detail.setDataStatus("0");//未处理
				businessFlowDetailMapper.insert(detail);
			}
		}
		
	}
}
