package com.chinaitop.depot.yylck.service.impl;

import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.profitlossStatement.mapper.StorageProfitlossStatementMapper;
import com.chinaitop.depot.utils.ParameterUtil;
import com.chinaitop.depot.yylck.mapper.StorageYylcktzdMapper;
import com.chinaitop.depot.yylck.model.StorageYylcktzd;
import com.chinaitop.depot.yylck.model.StorageYylcktzdExample;
import com.chinaitop.depot.yylck.service.StorageYylcktzdService;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class StorageYylcktzdServiceImpl implements StorageYylcktzdService {

	final static Logger log = LoggerFactory.getLogger(StorageYylcktzdServiceImpl.class);

	@Resource
	private StorageYylcktzdMapper storageYylcktzdMapper;

	@Resource
	private StorageProfitlossStatementMapper storageProfitlossStatementMapper;

	// 获取ip地址
    @Value("${crk-webservice}")
    private String url;

	@Override
	public List<StorageYylcktzd> getList(StorageYylcktzd storageYylcktzd, String ckkssj_str, String ckjssj_str) throws Exception {
		
		StorageYylcktzdExample example = new StorageYylcktzdExample();
		StorageYylcktzdExample.Criteria criteria = example.createCriteria();

		//仓房
		if (null != storageYylcktzd.getYhtch()) {
			criteria.andYhtchEqualTo(storageYylcktzd.getYhtch());
		}
		//货位
		if (null != storageYylcktzd.getYhthwh()) {
			criteria.andYhtchEqualTo(storageYylcktzd.getYhthwh());
		}
		//粮食品种
		if (null != storageYylcktzd.getYhtlspz()) {
			criteria.andYhtlspzEqualTo(storageYylcktzd.getYhtlspz());
		}
		//库ID
		if (null != storageYylcktzd.getOrgId()) {
			criteria.andOrgIdEqualTo(storageYylcktzd.getOrgId());
		}
		//数据状态(0:保存,1:提交)
		if (StringUtils.isNotBlank(storageYylcktzd.getDataStatus())) {
			criteria.andDataStatusEqualTo(storageYylcktzd.getDataStatus());
		}
		//通知单编号
		if (StringUtils.isNotBlank(storageYylcktzd.getTzdbh())) {
			criteria.andTzdbhLike("%"+storageYylcktzd.getTzdbh()+"%");
		}
		//原合同编号
		if (StringUtils.isNotBlank(storageYylcktzd.getYhtbh())) {
			criteria.andYhtbhLike("%"+storageYylcktzd.getYhtbh()+"%");
		}
		//客户名称
		if (StringUtils.isNotBlank(storageYylcktzd.getKhmc())) {
			criteria.andKhmcLike("%"+storageYylcktzd.getKhmc()+"%");
		}
		//溢余粮出库申请编号
		if (StringUtils.isNotBlank(storageYylcktzd.getYylcksqbh())) {
			criteria.andYylcksqbhLike("%"+storageYylcktzd.getYylcksqbh()+"%");
		}
		//出库开始时间
		if (StringUtils.isNotBlank(ckkssj_str)) {
			criteria.andCkkssjGreaterThanOrEqualTo(ParameterUtil.string2datetime(ckkssj_str));
		}
		//出库完成时间
		if (StringUtils.isNotBlank(ckjssj_str)) {
			criteria.andCkjssjLessThanOrEqualTo(ParameterUtil.string2datetime(ckjssj_str));
		}
		example.setOrderByClause(" cjsj desc");

		List<StorageYylcktzd> list = storageYylcktzdMapper.selectByExample(example);
		return list;
	}

	@Override
	public void saveOrUpdate(String yylcktzdObj) throws Exception {
		ObjectMapper objectMapper = new ObjectMapper();
		StorageYylcktzd storageYylcktzd = objectMapper.readValue(yylcktzdObj, StorageYylcktzd.class);

		int count = 0;
		if (StringUtils.isBlank(storageYylcktzd.getId())) {//新增
			storageYylcktzd.setId(UUID.randomUUID().toString().replace("-", ""));
			storageYylcktzd.setUpdatetime(storageYylcktzd.getCjsj());
			count = storageYylcktzdMapper.insert(storageYylcktzd);
		} else {//修改
			storageYylcktzd.setUpdatetime(new Date());
			count = storageYylcktzdMapper.updateByPrimaryKey(storageYylcktzd);
		}

		//如果是提交，那就给出入库推送过去
		if (count > 0 && "1".equals(storageYylcktzd.getDataStatus())) {
			pushYylcktzdData(storageYylcktzd);
		}
	}

	@Override
	public StorageYylcktzd findById(String id) throws Exception {
		StorageYylcktzd storageYylcktzd = storageYylcktzdMapper.selectByPrimaryKey(id);
		return storageYylcktzd;
	}

	@Override
	public void deleteObj(String id) throws Exception {
		StorageYylcktzd storageYylcktzd = findById(id);
		if (!"1".equals(storageYylcktzd.getDataStatus())) {
			storageYylcktzdMapper.deleteByPrimaryKey(id);
		}
	}

	/**
	 * 给出入库推送溢余粮出库通知单数据
	 * @throws Exception
	 */
	public void pushYylcktzdData(StorageYylcktzd storageYylcktzd) throws Exception {
		String url_ip = storageProfitlossStatementMapper.getCrkYjdAddress(storageYylcktzd.getOrgId().toString());
		 if (StringUtils.isNotBlank(url_ip)) {
			 //拼接完整URL
			 StringBuffer sbf_url = new StringBuffer(100);
			 sbf_url.append(url_ip).append(url);

			 //组装参数数据
			 JSONObject obj = new JSONObject();
			 obj.put("data", storageYylcktzd);
			 obj.put("message", "success");
			 obj.put("retCode", 200);

			 Object[] parameters = new Object[1];
			 String param = "["+JSON.toJSONString(obj)+"]";
			 parameters[0] = param;
			 

//			 log.info("溢余粮出库通知单推送地址是："+sbf_url.toString());
//			 log.info("溢余粮出库通知单推送参数是："+parameters);
//			 String str = createKC.createKC("createSyd", sbf_url.toString(), parameters);
//			 log.info("溢余粮出库通知单推送结果是："+str);
		 }
	}

	@Override
	public String getYylcktzdNo(Integer orgId) throws Exception {
		String yylcktzdNo = "";

		//YYCKTZD+年份+月份+日+两位流水号
		Date date = new Date();
		String day = ParameterUtil.date2string1(date);
		Date startTime = ParameterUtil.string2datetime(day+" 00:00:00");
		Date endTime = ParameterUtil.string2datetime(day+" 23:59:59");
		StorageYylcktzdExample example = new StorageYylcktzdExample();
		StorageYylcktzdExample.Criteria criteria = example.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andCjsjBetween(startTime, endTime);
		example.setOrderByClause("cjsj desc");
		List<StorageYylcktzd> list = storageYylcktzdMapper.selectByExample(example);
		StringBuffer sbf = new StringBuffer(50);
		if (null != list && list.size() > 0) {
			StorageYylcktzd storageYylcktzd = list.get(0);
			String no = storageYylcktzd.getYylcksqbh().substring(storageYylcktzd.getYylcksqbh().length()-2, storageYylcktzd.getYylcksqbh().length());
			Integer num = Integer.parseInt(no)+1;
			String sxm = String.format("%02d", num);
			sbf.append("YYCKTZD").append(ParameterUtil.date2string2(date)).append(sxm);
		} else {
			sbf.append("YYCKTZD").append(ParameterUtil.date2string2(date)).append("01");
		}
		yylcktzdNo = sbf.toString();

		return yylcktzdNo;
	}
}
